#include "header.hpp"

#include "pph21.amplitude.set.hpp"

void pph21_amplitude_initialization(munich * MUC){
  static Logger logger("pph21_amplitude_initialization");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  if (MUC->osi->switch_OL){
#ifdef OPENLOOPS
    MUC->asi = new pph21_amplitude_OpenLoops_set();
#endif
  }
  else if (MUC->osi->switch_RCL){
#ifdef RECOLA
    MUC->asi = new pph21_amplitude_Recola_set();
#endif
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


pph21_amplitude_set::pph21_amplitude_set(){
  Logger logger("pph21_amplitude_set::pph21_amplitude_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

pph21_amplitude_set::~pph21_amplitude_set(){
  Logger logger("pph21_amplitude_set::~pph21_amplitude_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


void pph21_amplitude_set::calculate_H2_QCD_2loop(){
  static Logger logger("pph21_amplitude_set::calculate_H2_QCD_2loop");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  //  pph41_calculate_H2(osi);
  osi->QT_A0 = osi->VA_b_ME2;
  double LQ = 2.0*log( osi->msi->M_H / osi->msi->M_t ); 

  osi->QT_H1_delta_QCD = (C_A*pi2 + 11)/2.0 ;
  osi->QT_H2_delta_QCD = 11399.0/144 + 19.0/8*LQ + 133.0*pi2/8 + 13.0/16*pi4 - 55.0/2*zeta3 + osi->N_f*(-1189.0/144 + 2.0*LQ/3 - 5.0*pi2/12); // formula from HNNLO (/Need/setup.f)
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


#ifdef OPENLOOPS

pph21_amplitude_OpenLoops_set::pph21_amplitude_OpenLoops_set(){
  Logger logger("pph21_amplitude_OpenLoops_set::pph21_amplitude_OpenLoops_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

pph21_amplitude_OpenLoops_set::~pph21_amplitude_OpenLoops_set(){
  Logger logger("pph21_amplitude_OpenLoops_set::~pph21_amplitude_OpenLoops_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

void pph21_amplitude_OpenLoops_set::calculate_ME2_VT2_QCD(){
  Logger logger("pph21_amplitude_OpenLoops_set::calculate_ME2_VT2_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  set_phasespacepoint();

  double mu_Q = (esi->p_parton[0][1] + esi->p_parton[0][2]).m();

  ol_setparameter_double(pole_uv, osi->VA_DeltaUV);
  ol_setparameter_double(pole_ir1, osi->VA_DeltaIR1);
  ol_setparameter_double(pole_ir2, osi->VA_DeltaIR2);

  int polenorm_0 = 0;
  ol_setparameter_int(polenorm, polenorm_0);

  ol_setparameter_double(OL_mu_ren, mu_Q);
  ol_setparameter_double(OL_mu_reg, mu_Q);
  ol_setparameter_double(fact_uv, one);
  ol_setparameter_double(fact_ir, one);

  //  ol_evaluate_loop2(1, P_OpenLoops, M2L2, &acc);
  ol_evaluate_full(1, P_OpenLoops, &osi->VA_b_ME2, M2L1, IRL1, M2L2, IRL2, &acc);
  ol_evaluate_ct(1, P_OpenLoops, &M2L0, &osi->VA_X_ME2);
  osi->VA_V_ME2 = M2L1[0] + osi->VA_X_ME2;
  // ??? One could simply use the sum as usually later...

  if (osi->initial_gg){
    osi->QT_H0_doublespinflip = 0.;
    osi->QT_H0_DG = 0.;
  }

  if (osi->switch_H2 || !osi->initial_diag){
    osi->QT_A0 = osi->VA_b_ME2;
    osi->QT_A1 = osi->VA_V_ME2;
    osi->QT_H1_delta_QCD = osi->QT_A1 / 2 / (osi->alpha_S * inv2pi * osi->QT_A0);
    osi->QT_A2 = 0.;
    osi->QT_H2_delta_QCD = 0.;
    //osi->QT_H2_delta_QCD = osi->QT_A2 / 4 / (pow(osi->alpha_S * inv2pi, 2) * osi->QT_A0);
  }
  else {
    calculate_H2_QCD_2loop();
  }
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

#endif


#ifdef RECOLA

pph21_amplitude_Recola_set::pph21_amplitude_Recola_set(){
  Logger logger("pph21_amplitude_Recola_set::pph21_amplitude_Recola_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

pph21_amplitude_Recola_set::~pph21_amplitude_Recola_set(){
  Logger logger("pph21_amplitude_Recola_set::~pph21_amplitude_Recola_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

#endif
