#include "header.hpp"

// 2-loop helicity amplitude coefficients for qq_bar -> Wgam/Zgam,
// see T. Gehrmann, L. Tancredi, 1112.1531

#include "pplla03.amplitude.set.hpp"

void pplla03_amplitude_set::Omega_2_W(double e_q, double e_qprime){
  //, double_complex & alpha_W, double_complex & beta_W, double_complex & gamma_W
  //  double_complex Omega1, Omega2, Omega3;
  //  int n_f = 5;

  alpha_2_W = e_qprime * alpha_2_1() + e_q * alpha_2_2() + alpha_2_3();
  beta_2_W = e_qprime * beta_2_1() + e_q * beta_2_2() + beta_2_3();
  gamma_2_W = e_qprime * gamma_2_1() + e_q * gamma_2_2() + gamma_2_3();

  /*
    alpha_2_1(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega1);
    alpha_2_2(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega2);
    alpha_2_3(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega3);
    //  cout << "alpha: " << Omega1 << ", " << Omega2 << ", " << Omega4 << endl;
    alpha_W = e_qprime * Omega1 + e_q * Omega2 + Omega3;

    beta_2_1(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega1);
    beta_2_2(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega2);
    beta_2_3(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega3);
    //  cout << "alpha: " << Omega1 << ", " << Omega2 << ", " << Omega4 << endl;
    beta_W = e_qprime * Omega1 + e_q * Omega2 + Omega3;

    gamma_2_1(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega1);
    gamma_2_2(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega2);
    gamma_2_3(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega3);
    //  cout << "alpha: " << Omega1 << ", " << Omega2 << ", " << Omega4 << endl;
    gamma_W = e_qprime * Omega1 + e_q * Omega2 + Omega3;

    //  cout << "alpha: " << alpha_ZR << ", " << alpha_ZL << ", " << alpha_P << endl;
    */
  cout << "alpha: " << alpha_2_W << ", " << beta_2_W << ", " << gamma_2_W << endl;
}

void pplla03_amplitude_set::Omega_2_ZP(double e_q){
  //, double_complex & alpha_ZR, double_complex & beta_ZR, double_complex & gamma_ZR, double_complex & alpha_ZL, double_complex & beta_ZL, double_complex & gamma_ZL, double_complex & alpha_P, double_complex & beta_P, double_complex & gamma_P
  //  double_complex Omega1, Omega2, Omega4;
  //  int n_f = 5;

  double I3_q;
  if (e_q > 0) {
    I3_q = 0.5;
  } else {
    I3_q = -0.5;
  }

  //  double_complex cos_W = sqrt(1.0 - sin_W * sin_W);

  double_complex RZ_q1q2 = -msi->csin_w * e_q / msi->ccos_w;
  double_complex LZ_q1q2 = (I3_q - msi->csin2_w * e_q) / (msi->csin_w * msi->ccos_w);

  double_complex N_FZR = (-1.0 / 3.0 * 3.0 * ((-0.5 + msi->csin2_w / 3.0) / msi->csin_w / msi->ccos_w + 1.0 / 3.0 * msi->csin_w / msi->ccos_w) + 2.0 / 3.0 * 2.0 * ((0.5 - msi->csin2_w * 2.0 / 3.0) / msi->csin_w / msi->ccos_w - 2.0 / 3.0 * msi->csin_w / msi->ccos_w)) / (2.0 * RZ_q1q2); // TODO: check!
  double_complex N_FZL = (-1.0 / 3.0 * 3.0 * ((-0.5 + msi->csin2_w / 3.0) / msi->csin_w / msi->ccos_w + 1.0 / 3.0 * msi->csin_w / msi->ccos_w) + 2.0 / 3.0 * 2.0 * ((0.5 - msi->csin2_w * 2.0 / 3.0) / msi->csin_w / msi->ccos_w - 2.0 / 3.0 * msi->csin_w / msi->ccos_w)) / (2.0 * LZ_q1q2); // TODO: check!
  double N_FP = 11.0 / 9 / e_q; // TODO: check!

  //  cout << N_FZR << ", " << N_FZL << ", " << N_FP << endl;

  alpha_2_ZR = e_q * (alpha_2_1() + alpha_2_2()) + N_FZR * alpha_2_4();
  alpha_2_ZL = e_q * (alpha_2_1() + alpha_2_2()) + N_FZL * alpha_2_4();
  alpha_2_P = e_q * (alpha_2_1() + alpha_2_2()) + N_FP * alpha_2_4();

  beta_2_ZR = e_q * (beta_2_1() + beta_2_2()) + N_FZR * beta_2_4();
  beta_2_ZL = e_q * (beta_2_1() + beta_2_2()) + N_FZL * beta_2_4();
  beta_2_P = e_q * (beta_2_1() + beta_2_2()) + N_FP * beta_2_4();

  gamma_2_ZR = e_q * (gamma_2_1() + gamma_2_2()) + N_FZR * gamma_2_4();
  gamma_2_ZL = e_q * (gamma_2_1() + gamma_2_2()) + N_FZL * gamma_2_4();
  gamma_2_P = e_q * (gamma_2_1() + gamma_2_2()) + N_FP * gamma_2_4();

  /*
    alpha_2_1(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega1);
    alpha_2_2(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega2);
    alpha_2_4(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega4);
    //  cout << "alpha: " << Omega1 << ", " << Omega2 << ", " << Omega4 << endl;
    alpha_ZR = e_q * (Omega1 + Omega2) + N_FZR * Omega4;
    alpha_ZL = e_q * (Omega1 + Omega2) + N_FZL * Omega4;
    alpha_P = e_q * (Omega1 + Omega2) + N_FP * Omega4;

    //  cout << "alpha: " << alpha_ZR << ", " << alpha_ZL << ", " << alpha_P << endl;

    beta_2_1(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega1);
    beta_2_2(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega2);
    beta_2_4(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega4);
    //  cout << "beta: " << Omega1 << ", " << Omega2 << ", " << Omega4 << endl;
    beta_ZR = e_q * (Omega1 + Omega2) + N_FZR * Omega4;
    beta_ZL = e_q * (Omega1 + Omega2) + N_FZL * Omega4;
    beta_P = e_q * (Omega1 + Omega2) + N_FP * Omega4;

    //  cout << "beta: " << beta_ZR << ", " << beta_ZL << ", " << beta_P << endl;

    gamma_2_1(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega1);
    gamma_2_2(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega2);
    gamma_2_4(u, v, n_f, GHPLs1, GHPLs2, GHPLs3, GHPLs4, HPLs1, HPLs2, HPLs3, HPLs4, Omega4);
    //  cout << "gamma: " << Omega1 << ", " << Omega2 << ", " << Omega4 << endl;
    gamma_ZR = e_q * (Omega1 + Omega2) + N_FZR * Omega4;
    gamma_ZL = e_q * (Omega1 + Omega2) + N_FZL * Omega4;
    gamma_P = e_q * (Omega1 + Omega2) + N_FP * Omega4;

    //  cout << "gamma: " << gamma_ZR << ", " << gamma_ZL << ", " << gamma_P << endl;
    */
}
