#include "header.hpp"

#include "ppllll04.amplitude.set.hpp"

void ppllll04_amplitude_set::calculate_H2_QCD_2loop_computeVVprimeHelAmplitudesDR(int i_x, int i_a){
  static Logger logger("ppllll04_amplitude_set::calculate_H2_QCD_2loop_computeVVprimeHelAmplitudesDR");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  // propagators for the decay
  double_complex DV1;
  if (DR[i_x][i_a].V1 == V_PHOT){DV1 = ma2;}
  else if (DR[i_x][i_a].V1 == V_ZLL || DR[i_x][i_a].V1 == V_ZNN){DV1 = ma2 - msi->cM2_Z;}
  else if (DR[i_x][i_a].V1 == V_WMIN || DR[i_x][i_a].V1 == V_WPLU){DV1 = ma2 - msi->cM2_W;}
  double_complex DV2;
  if (DR[i_x][i_a].V2 == V_PHOT){DV2 = mb2;}
  else if (DR[i_x][i_a].V2 == V_ZLL || DR[i_x][i_a].V2 == V_ZNN) {DV2 = mb2 - msi->cM2_Z;}
  else if (DR[i_x][i_a].V2 == V_WMIN || DR[i_x][i_a].V2 == V_WPLU) {DV2 = mb2 - msi->cM2_W;}

  // propagator denominators for the F_V contributions
  double_complex DP = s;
  double_complex DZ = s - msi->cM2_Z;
  double_complex DW = s - msi->cM2_W;

  double_complex Q_FP[2];
  Q_FP[0] = DR[i_x][i_a].L_P * DR[i_x][i_a].c_P12 / DP;
  Q_FP[1] = DR[i_x][i_a].R_P * DR[i_x][i_a].c_P12 / DP;
  double_complex Q_FZ[2];
  Q_FZ[0] = DR[i_x][i_a].L_Z * DR[i_x][i_a].c_Z12 / DZ;
  Q_FZ[1] = DR[i_x][i_a].R_Z * DR[i_x][i_a].c_Z12 / DZ;
  double_complex Q_FW[2];
  Q_FW[0] = DR[i_x][i_a].L_W * DR[i_x][i_a].c_W12 / DW;
  Q_FW[1] = DR[i_x][i_a].R_W * DR[i_x][i_a].c_W12 / DW;

  double_complex M_undressed[2][2][2][3][4];
  for (int loop=0; loop<3; loop++) {
    for (int type=0; type<4; type++) {
      M_undressed[0][0][0][loop][type] = compute_M(1, 2, 5, 6, 7, 8, -1, loop, type);
      M_undressed[0][0][1][loop][type] = compute_M(1, 2, 5, 6, 8, 7, -1, loop, type);
      M_undressed[0][1][0][loop][type] = compute_M(1, 2, 6, 5, 7, 8, -1, loop, type);
      M_undressed[0][1][1][loop][type] = compute_M(1, 2, 6, 5, 8, 7, -1, loop, type);
      M_undressed[1][0][0][loop][type] = compute_M(1, 2, 5, 6, 7, 8, +1, loop, type);
      M_undressed[1][0][1][loop][type] = compute_M(1, 2, 5, 6, 8, 7, +1, loop, type);
      M_undressed[1][1][0][loop][type] = compute_M(1, 2, 6, 5, 7, 8, +1, loop, type);
      M_undressed[1][1][1][loop][type] = compute_M(1, 2, 6, 5, 8, 7, +1, loop, type);
    }
  }
  
  for (int loop=0; loop<3; loop++) {
    for (int h1=0; h1<2; h1++) {
      for (int h2=0; h2<2; h2++) {
        for (int h3=0; h3<2; h3++) {
	  /*
	  cout << "Q_A[" << h1 << "] = " << DR[i_x][i_a].Q_A[h1] << "   Q_B[" << h1 << "] = " << DR[i_x][i_a].Q_B[h1] << "   Q_C[" << h1 << "] = " << DR[i_x][i_a].Q_C[h1] << endl;
	  cout << "Q_FP[" << h1 << "] = " << Q_FP[h1] << "   Q_FZ[" << h1 << "] = " << Q_FZ[h1] << "   Q_FW[" << h1 << "] = " << Q_FW[h1] << endl;
	  cout << "LR56[" << h2 << "] = " << DR[i_x][i_a].LR56[h2] << "   LR78[" << h3 << "] = " << DR[i_x][i_a].LR78[h3] << endl;
	  for (int type=0; type<4; type++) {
	    cout << "M_undressed[" << h1 << "][" << h2 << "][" << h3 << "][" << loop << "][" << type << "] = " << M_undressed[h1][h2][h3][loop][type] << endl;
	  }
	  */
	  M_dressed_tmp[h1][h2][h3][loop] = DR[i_x][i_a].Q_A[h1] * M_undressed[h1][h2][h3][loop][0];
          M_dressed_tmp[h1][h2][h3][loop] += DR[i_x][i_a].Q_B[h1] * M_undressed[h1][h2][h3][loop][1];
          M_dressed_tmp[h1][h2][h3][loop] += DR[i_x][i_a].Q_C[h1] * M_undressed[h1][h2][h3][loop][2];
          M_dressed_tmp[h1][h2][h3][loop] += (Q_FP[h1] + Q_FZ[h1] + Q_FW[h1]) * M_undressed[h1][h2][h3][loop][3];
          M_dressed_tmp[h1][h2][h3][loop] *= (DR[i_x][i_a].LR56[h2] * DR[i_x][i_a].LR78[h3] / DV1 / DV2);

	  //	  cout << "M_dressed_tmp[" << h1 << "][" << h2 << "][" << h3 << "][" << loop << "] = " << M_dressed_tmp[h1][h2][h3][loop] << endl;
	  
          M_dressed_tmp[h1][h2][h3][loop] *= double_complex(0,1);
        }
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


double_complex ppllll04_amplitude_set::compute_M(int i1, int i2, int i5, int i6, int i7, int i8, int hel, int loop, int type){
  static Logger logger("ppllll04_amplitude_set::compute_M");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;
  
  i5 -= 2;
  i6 -= 2;
  i7 -= 2;
  i8 -= 2;

  double_complex result = 0.;
  result += E[loop][type][0] * za[i1][i5] * za[i1][i7] * zb[i1][i6] * zb[i1][i8];
  result += E[loop][type][1] * za[i1][i5] * za[i2][i7] * zb[i1][i6] * zb[i2][i8];
  result += E[loop][type][2] * za[i2][i5] * za[i1][i7] * zb[i2][i6] * zb[i1][i8];
  result += E[loop][type][3] * za[i2][i5] * za[i2][i7] * zb[i2][i6] * zb[i2][i8];
  result += E[loop][type][4] * za[i5][i7] * zb[i6][i8];
  if (hel == -1) {
    result *= (zb[i1][i5] * za[i5][i2]+zb[i1][i6] * za[i6][i2]);
  }
  else if (hel == +1) {
    result *= (zb[i2][i5] * za[i5][i1]+zb[i2][i6] * za[i6][i1]);
  }

  if (hel == -1) {
    result += E[loop][type][5] * za[i1][i5] * za[i2][i7] * zb[i1][i6] * zb[i1][i8];
    result += E[loop][type][6] * za[i2][i5] * za[i2][i7] * zb[i2][i6] * zb[i1][i8];
    result += E[loop][type][7] * za[i2][i5] * za[i1][i7] * zb[i1][i6] * zb[i1][i8];
    result += E[loop][type][8] * za[i2][i5] * za[i2][i7] * zb[i1][i6] * zb[i2][i8];
  }
  else if (hel == +1) {
    result += E[loop][type][5] * za[i1][i5] * za[i1][i7] * zb[i1][i6] * zb[i2][i8];
    result += E[loop][type][6] * za[i2][i5] * za[i1][i7] * zb[i2][i6] * zb[i2][i8];
    result += E[loop][type][7] * za[i1][i5] * za[i1][i7] * zb[i2][i6] * zb[i1][i8];
    result += E[loop][type][8] * za[i1][i5] * za[i2][i7] * zb[i2][i6] * zb[i2][i8];
  }
  /*
  if (i5 == 3 && i6 == 4 &&i7 == 5 && i8 == 6 && hel == -1) {
//  cout << "altern=" << -4.0 * za[i5][i2] * zb[i1][i8] * (zb[i1][i6] * za[i7][i1]-zb[i8][i6] * za[i7][i8])/(-za[i1][i7] * zb[i7][i1]-za[i1][i8] * zb[i8][i1]+za[i7][i8] * zb[i8][i7]) << endl;
//  cout << "altern=" << -4.0 * za[i7][i2] * zb[i1][i6] * (zb[i1][i8] * za[i5][i1]-zb[i6][i8] * za[i5][i6])/(-za[i1][i5] * zb[i5][i1]-za[i1][i6] * zb[i6][i1]+za[i5][i6] * zb[i6][i5]) << endl;
  }
  */

  logger << LOG_DEBUG_VERBOSE << "finished - return " << result << endl;
  return result;
}


//#include "old.ppllll04.amplitude.H2.QCD.helicity.amplitudes.DR.cpp"
