#include "header.hpp"

#include "ppllll04.amplitude.gg.set.hpp"

void ppllll04_amplitude_gg_set::initialization_H1gg_QCD_2loop(){
  static Logger logger("ppllll04_amplitude_gg_set::initialization_H1gg_QCD_2loop");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  q1 = 0;

  M_gg_dressed.resize(2, vector<vector<vector<vector<double_complex> > > > (2, vector<vector<vector<double_complex> > > (2, vector<vector<double_complex> > (2,vector<double_complex> (3, 0.)))));
  M_gg_dressed2.resize(2, vector<vector<vector<vector<double_complex> > > > (2, vector<vector<vector<double_complex> > > (2, vector<vector<double_complex> > (2,vector<double_complex> (3, 0.)))));
  M_gg_dressed_tmp.resize(2, vector<vector<vector<vector<double_complex> > > > (2, vector<vector<vector<double_complex> > > (2, vector<vector<double_complex> > (2,vector<double_complex> (3, 0.)))));

  p_VVamp.resize(7);
  za.resize(7, vector<double_complex> (7, 0.));
  zb.resize(7, vector<double_complex> (7, 0.));
  s_inv.resize(7, vector<double> (7, 0.));

  sym_factor = 1. / csi->symmetry_factor;

  E.resize(2, vector<vector<double_complex> >(2, vector<double_complex> (9)));

  // compute dressed amplitudes for all contributions to respective process
  if (osi->csi->subprocess == "gg_emmumepmup"){
    permutation.resize(1);
    DR.resize(1);
    SR.resize(1);
    permutation[0] = {0,1,2,3,5,4,6};
    // ZZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_ZLL, V_ZLL, this, permutation[0]));
    // gamgam contribution
    DR[0].push_back(ppllll04_decay_DR(V_PHOT, V_PHOT, this, permutation[0]));
    // gamZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_PHOT, V_ZLL, this, permutation[0]));
    // Zgam contribution
    DR[0].push_back(ppllll04_decay_DR(V_ZLL, V_PHOT, this, permutation[0]));
  }
  else if (osi->csi->subprocess == "gg_ememepep"){
    permutation.resize(2);
    DR.resize(2);
    SR.resize(2);
    permutation[0] = {0,1,2,3,5,4,6};
    // ZZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_ZLL, V_ZLL, this, permutation[0]));
    // gamgam contribution
    DR[0].push_back(ppllll04_decay_DR(V_PHOT, V_PHOT, this, permutation[0]));
    // gamZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_PHOT, V_ZLL, this, permutation[0]));
    // Zgam contribution
    DR[0].push_back(ppllll04_decay_DR(V_ZLL, V_PHOT, this, permutation[0]));
    // same flavour case: take into account additional crossings
    permutation[1] = {0,1,2,3,6,4,5};
    // ZZ contribution
    DR[1].push_back(ppllll04_decay_DR(V_ZLL, V_ZLL, this, permutation[1]));
    // gamgam contribution
    DR[1].push_back(ppllll04_decay_DR(V_PHOT, V_PHOT, this, permutation[1]));
    // gamZ contribution
    DR[1].push_back(ppllll04_decay_DR(V_PHOT, V_ZLL, this, permutation[1]));
    // Zgam contribution
    DR[1].push_back(ppllll04_decay_DR(V_ZLL, V_PHOT, this, permutation[1]));
  }
  else if (osi->csi->subprocess == "gg_emmupvmve~"){
    permutation.resize(1);
    DR.resize(1);
    SR.resize(1);
    permutation[0] = {0,1,2,3,6,5,4};
    // WW contribution
    DR[0].push_back(ppllll04_decay_DR(V_WMIN, V_WPLU, this, permutation[0]));
  }
  else if (osi->csi->subprocess == "gg_emepvmvm~"){
    permutation.resize(1);
    DR.resize(1);
    SR.resize(1);
    permutation[0] = {0,1,2,3,4,5,6};
    // ZZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_ZLL, V_ZNN, this, permutation[0]));
   // gamZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_PHOT, V_ZNN, this, permutation[0]));
  }
  else if (osi->csi->subprocess == "gg_emepveve~"){
    permutation.resize(2);
    DR.resize(2);
    SR.resize(2);
    permutation[0] = {0,1,2,3,4,5,6};
    // ZZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_ZLL, V_ZNN, this, permutation[0]));
    // gamZ contribution
    DR[0].push_back(ppllll04_decay_DR(V_PHOT, V_ZNN, this, permutation[0]));
    // same flavour case: take into account additional crossings
    permutation[1] = {0,1,2,3,6,5,4};
    // WW contribution
    DR[1].push_back(ppllll04_decay_DR(V_WMIN, V_WPLU, this, permutation[1]));
  }
  else {
    assert(false);
  }

  ggVVprime::getInstance().initAmplitude(osi->N_f);

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


void ppllll04_amplitude_gg_set::add_hel_amplitudes_H1gg_QCD_2loop(int i_x){
  static Logger logger("ppllll04_amplitude_gg_set::add_hel_amplitudes_H1gg_QCD_2loop");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;
  
  for (int h1 = 0; h1 < 2; h1++){
    for (int h2 = 0; h2 < 2; h2++){
      for (int h3 = 0; h3 < 2; h3++){
	for (int h4 = 0; h4 < 2; h4++){
	  for (int loop = 0; loop < 3; loop++){
	    if (i_x == 0){M_gg_dressed[h1][h2][h3][h4][loop] += M_gg_dressed_tmp[h1][h2][h3][h4][loop];}
	    else {M_gg_dressed2[h1][h2][h3][h4][loop] += M_gg_dressed_tmp[h1][h2][h3][h4][loop];}
	  }
	}
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


void ppllll04_amplitude_gg_set::reset_hel_amplitudes_H1gg_QCD_2loop(int i_x){
  static Logger logger("ppllll04_amplitude_gg_set::reset_hel_amplitudes_H1gg_QCD_2loop");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;
  
  for (int h1 = 0; h1 < 2; h1++){
    for (int h2 = 0; h2 < 2; h2++){
      for (int h3 = 0; h3 < 2; h3++){
	for (int h4 = 0; h4 < 2; h4++){
	  for (int loop = 0; loop < 3; loop++){
	    if (i_x == 0){M_gg_dressed[h1][h2][h3][h4][loop] = 0.;}
	    else {M_gg_dressed2[h1][h2][h3][h4][loop] = 0.;}
	  }
	}
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

