#include "header.hpp"

#include "ppllll04.amplitude.gg.set.hpp"

void ppllll04_amplitude_gg_initialization(munich * MUC){
  static Logger logger("ppllll04_amplitude_gg_initialization");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  if (MUC->osi->switch_OL){
#ifdef OPENLOOPS
    MUC->asi = new ppllll04_amplitude_gg_OpenLoops_set();
#endif
  }
  else if (MUC->osi->switch_RCL){
#ifdef RECOLA
    MUC->asi = new ppllll04_amplitude_gg_Recola_set();
#endif
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


ppllll04_amplitude_gg_set::ppllll04_amplitude_gg_set(){
  Logger logger("ppllll04_amplitude_gg_set::ppllll04_amplitude_gg_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_amplitude_gg_set::~ppllll04_amplitude_gg_set(){
  Logger logger("ppllll04_amplitude_gg_set::~ppllll04_amplitude_gg_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

void ppllll04_amplitude_gg_set::calculate_H1gg_QCD_2loop(){
  static Logger logger("ppllll04_amplitude_gg_set::calculate_H1gg_2loop");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  //  ggllll34_calculate_H1(osi);
  /////  logger << LOG_DEBUG_VERBOSE << "NEW VT2 implementation called" << endl;
  //  ggllll34_calculate_amplitude_doublevirtual(osi);
  /////  calculate_H1gg_QCD_2loop_amplitude_doublevirtual();

  //  static bool debug = Log::getLogThreshold()<=LOG_DEBUG; 
  
  for (int i_x = 0; i_x < permutation.size(); i_x++){
    for (int i_p = 0; i_p < permutation[i_x].size(); i_p++){
      p_VVamp[i_p] = esi->p_parton[0][permutation[i_x][i_p]];
    }
    s = (p_VVamp[1] + p_VVamp[2]).m2();
    t = (p_VVamp[1] - (p_VVamp[3] + p_VVamp[4])).m2();
    ma2 = (p_VVamp[3] + p_VVamp[4]).m2();
    mb2 = (p_VVamp[5] + p_VVamp[6]).m2();
    logger << LOG_DEBUG << "sqrt(s)=" << sqrt(s) << ", sqrt(-t)=" << sqrt(-t) << ", sqrt(-u)=" << sqrt(s + t - ma2 - mb2) << ", ma=" << sqrt(ma2) << ", mb=" << sqrt(mb2) << endl;

    calculate_spinorproducts(p_VVamp, za, zb, s_inv);
    //   calcSpinorProducts(p_VVamp, za, zb, s_inv);

    ggVVprime::getInstance().computeAmplitude(s, t, ma2, mb2);
    for (int loop = 0; loop < 2; loop++){
      for (int hel = 0; hel < 2; hel++){
	for (int i = 1; i <= 9; i++){
	  double Er,Ei;
	  ggVVprime::getInstance().getAmplitude(loop,i,hel,Er,Ei);
	  E[loop][hel][i-1] = double_complex(Er,Ei);
	}
      }
    }

    reset_hel_amplitudes_H1gg_QCD_2loop(i_x);
 
    for (int i_a = 0; i_a < DR[i_x].size(); i_a++){
      calculate_H1gg_QCD_2loop_computeVVprimeHelAmplitudes(i_x, i_a);
      add_hel_amplitudes_H1gg_QCD_2loop(i_x);
    }
  }

  // square and add up helicity amplitudes
  osi->QT_A0 = 0;
  osi->QT_A1 = 0;
  for (int h1 = 0; h1 < 2; h1++) {
    for (int h2 = 0; h2 < 2; h2++) {
      for (int h3 = 0; h3 < 2; h3++) {
	for (int h4 = 0; h4 < 2; h4++) {
	  osi->QT_A0 += norm(M_gg_dressed[h1][h2][h3][h4][0]) + norm(M_gg_dressed2[h1][h2][h3][h4][0]);
	  osi->QT_A1 += 2 * real(M_gg_dressed[h1][h2][h3][h4][0] * conj(M_gg_dressed[h1][h2][h3][h4][1])) + 2 * real(M_gg_dressed2[h1][h2][h3][h4][0] * conj(M_gg_dressed2[h1][h2][h3][h4][1]));
	  if (h3 == h4) {
	    osi->QT_A0 -= 2 * real(conj(M_gg_dressed[h1][h2][h3][h4][0]) * M_gg_dressed2[h1][h2][h3][h4][0]);
          
	    osi->QT_A1 -= 2 * real(conj(M_gg_dressed[h1][h2][h3][h4][0]) * M_gg_dressed2[h1][h2][h3][h4][1]);
	    osi->QT_A1 -= 2 * real(conj(M_gg_dressed2[h1][h2][h3][h4][0]) * M_gg_dressed[h1][h2][h3][h4][1]);
	  }
        }
      }
    }
  }

  double alpha_e = osi->msi->alpha_e;

  osi->QT_A0 *= pow(4*M_PI*alpha_e,4);
  osi->QT_A0 /= (4*64); // averaging factor
  osi->QT_A0 *= 2; // color factor 1/2 d^AB * 1/2 d^AB
  osi->QT_A0 *= sym_factor;

  osi->QT_A1 *= pow(4*M_PI*alpha_e,4);
  osi->QT_A1 /= (4*64); // averaging factor
  osi->QT_A1 *= 2; // color factor 1/2 d^AB * 1/2 d^AB
  osi->QT_A1 *= sym_factor;

  // multiply with alpha_s^2 1-loop LO and alpha_s^3 2-loop NLO
  osi->QT_A0 *= pow(osi->alpha_S/2./M_PI,2);
  osi->QT_A1 *= pow(osi->alpha_S/2./M_PI,3);

  //  cout << pow(osi->alpha_S/2./M_PI,2) << endl;
  //  cout << osi->alpha_S << endl;

  // no idea why there is a factor of 4 missing:
  osi->QT_A0 *= 4.;
  osi->QT_A1 *= 4.;

  // Born reweighting procedure should take place here, instead of the general L2VA/L2VT implememtations !!!
  
  // Different cases:
  // L2VA does not use  osi->QT_H1_delta_QCD  directly, but contructs  osi->VA_V_ME2  from
  // osi->QT_A1.

  // osi->QT_H1_delta_QCD may not containt the factor (osi->VA_b_ME2 / osi->QT_A0) since
  // it is multiplied with osi->VA_b_ME2 anyway, which automatically does the reweighting !!!
  //  osi->QT_A1 *= (osi->VA_b_ME2 / osi->QT_A0);
  osi->VA_V_ME2 = osi->QT_A1 * (osi->VA_b_ME2 / osi->QT_A0);

  logger << LOG_DEBUG_POINT << setprecision(15) << "<pre>:  ratio = " << setw(23) << osi->QT_A0 / osi->VA_b_ME2 << "   A0 = " << setw(23) << osi->QT_A0 << "   A0 = " << setw(23) << "   b_ME2 = " << osi->VA_b_ME2 << endl;
  /*
  if (csi->type_contribution == "L2VA"){
    logger << LOG_DEBUG_POINT << "ratios = " << osi->QT_A0 / osi->VA_b_ME2 << endl;

    logger << LOG_DEBUG_POINT << "born VVamp = " << setprecision(15) << setw(23) << osi->QT_A0 << " ,   "
	   << "1-loop VVamp = " << setprecision(15) << setw(23) << osi->QT_A1 << endl;

    // reweighting 2-loop amplitude with mt-dependence (from osi->VA_b_ME2):
    osi->VA_V_ME2 = osi->QT_A1 * osi->VA_b_ME2 / osi->QT_A0;

    // no mt-dependence (from osi->VA_b_ME2) in 2-loop amplitude:
    //      osi->VA_V_ME2 = osi->QT_A1;

    // osi.QT_A1 is only the two-loop amplitude (times one-loop...),
    // without reweighting with any loop² result
    logger << LOG_DEBUG_POINT << "born OL    = " << setprecision(15) << setw(23) << osi->VA_b_ME2 << " ,   "
	   << "             = " << setprecision(15) << setw(23) << osi->VA_V_ME2 << endl;
    logger << LOG_DEBUG_POINT << "osi->VA_V_ME2 = " << osi->VA_V_ME2 << endl;
  }
  */
  // L2VT uses  osi->QT_H1_delta_QCD , which is a relative factor to the Born amplitude.
  // Consequently, the reweighting happens "automatically" since
  // osi->QT_H1_delta_QCD ~ osi->QT_A1 / osi->QT_A0 , which is later multiplied by
  // osi->VA_b_ME2.
  if (csi->type_contribution == "L2VT"){
    osi->QT_H1_delta_QCD = osi->QT_A1;
    osi->QT_H1_delta_QCD /= 2;
    osi->QT_H1_delta_QCD /= (osi->alpha_S * inv2pi * osi->QT_A0);
    logger << LOG_DEBUG_POINT << "osi->QT_H1_delta_QCD = " << osi->QT_H1_delta_QCD << endl;
  }
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


#ifdef OPENLOOPS

ppllll04_amplitude_gg_OpenLoops_set::ppllll04_amplitude_gg_OpenLoops_set(){
  Logger logger("ppllll04_amplitude_gg_OpenLoops_set::ppllll04_amplitude_gg_OpenLoops_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_amplitude_gg_OpenLoops_set::~ppllll04_amplitude_gg_OpenLoops_set(){
  Logger logger("ppllll04_amplitude_gg_OpenLoops_set::~ppllll04_amplitude_gg_OpenLoops_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


void ppllll04_amplitude_gg_OpenLoops_set::initialization_process(){
  Logger logger("ppllll04_amplitude_gg_OpenLoops_set::initialization_process");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  if (csi->type_contribution == "VT2" && csi->type_correction == "QCD"){
    initialization_H2_QCD_2loop();
  }
  if ((csi->type_contribution == "L2VA" || csi->type_contribution == "L2VT") && csi->type_correction == "QCD"){
    initialization_H1gg_QCD_2loop();
  }
  amplitude_OpenLoops_set::initialization_process();
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

#endif


#ifdef RECOLA

ppllll04_amplitude_gg_Recola_set::ppllll04_amplitude_gg_Recola_set(){
  Logger logger("ppllll04_amplitude_gg_Recola_set::ppllll04_amplitude_gg_Recola_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_amplitude_gg_Recola_set::~ppllll04_amplitude_gg_Recola_set(){
  Logger logger("ppllll04_amplitude_gg_Recola_set::~ppllll04_amplitude_gg_Recola_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


void ppllll04_amplitude_gg_Recola_set::initialization_process(){
  Logger logger("ppllll04_amplitude_gg_Recola_set::initialization_process");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  if (csi->type_contribution == "VT2" && csi->type_correction == "QCD"){
    initialization_H2_QCD_2loop();
  }
  if ((csi->type_contribution == "L2VA" || csi->type_contribution == "L2VT") && csi->type_correction == "QCD"){
    initialization_H1gg_QCD_2loop();
  }
  amplitude_Recola_set::initialization_process();
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


#endif
