#include "header.hpp"

#include "ppllll04.amplitude.set.hpp"
//#include "ppllll24.amplitude.doublevirtual.hpp"
//#include "ggllll34.amplitude.doublevirtual.hpp"

void ppllll04_amplitude_initialization(munich * MUC){
  static Logger logger("ppllll04_amplitude_initialization");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  if (MUC->osi->switch_OL){
#ifdef OPENLOOPS
    MUC->asi = new ppllll04_amplitude_OpenLoops_set();
#endif
  }
  else if (MUC->osi->switch_RCL){
#ifdef RECOLA
    MUC->asi = new ppllll04_amplitude_Recola_set();
#endif
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


ppllll04_amplitude_set::ppllll04_amplitude_set(){
  Logger logger("ppllll04_amplitude_set::ppllll04_amplitude_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_amplitude_set::~ppllll04_amplitude_set(){
  Logger logger("ppllll04_amplitude_set::~ppllll04_amplitude_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

void ppllll04_amplitude_set::calculate_H2_QCD_2loop(){
  static Logger logger("ppllll04_amplitude_set::calculate_H2_QCD_2loop");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  ///  calculate_H2_QCD_2loop_initialization_ingredients();
  
  //  ppllll24_calculate_H2(osi);
  //  logger << LOG_DEBUG_VERBOSE << "NEW VT2 implementation called" << endl;
  //  ppllll24_calculate_amplitude_doublevirtual(osi);
  //  calculate_H2_QCD_2loop_amplitude_doublevirtual();
  //  calculate_H2_QCD_2loop_helicity_amplitude();

  /*
  int static init = 1;
  if (init == 1) {
    // initialize the form factor library
    VVprime::getInstance().initAmplitude(osi->N_f);
    init = 0;
  }
  */
  static bool debug = Log::getLogThreshold()<=LOG_DEBUG; 

  for (int i_x = 0; i_x < permutation.size(); i_x++){
    for (int i_p = 0; i_p < permutation[i_x].size(); i_p++){
      p_VVamp[i_p] = esi->p_parton[0][permutation[i_x][i_p]];
    }
    s = (p_VVamp[1] + p_VVamp[2]).m2();
    t = (p_VVamp[1] - (p_VVamp[3] + p_VVamp[4])).m2();
    ma2 = (p_VVamp[3] + p_VVamp[4]).m2();
    mb2 = (p_VVamp[5] + p_VVamp[6]).m2();
    logger << LOG_DEBUG << "sqrt(s)=" << sqrt(s) << ", sqrt(-t)=" << sqrt(-t) << ", sqrt(-u)=" << sqrt(s + t - ma2 - mb2) << ", ma=" << sqrt(ma2) << ", mb=" << sqrt(mb2) << endl;
    /*
  double s = (p_VVamp[1] + p_VVamp[2]).m2();
  //  double t = (p_VVamp[1] - (p_VVamp[3] + p_VVamp[4])).m2();
  double ma2 = (p_VVamp[3] + p_VVamp[4]).m2();
  double mb2 = (p_VVamp[5] + p_VVamp[6]).m2();

  vector<vector<double_complex > > za, zb;
  vector<vector<double> > s_inv;
    */
    calculate_spinorproducts(p_VVamp, za, zb, s_inv);
    //   calcSpinorProducts(p_VVamp, za, zb, s_inv);

    VVprime::getInstance().computeAmplitude(s, t, ma2, mb2, debug);
    for (int loop = 0; loop < 3; loop++) {
      for (int type = 0; type < 4; type++) {
	//       ppllll24_computeE(s,t,ma2,mb2,type,E);
	for (int i = 1; i <= 9; i++) {
	  double Er,Ei;
	  VVprime::getInstance().getAmplitude(loop,i,type,Er,Ei);
	  E[loop][type][i-1] = double_complex(Er,Ei);
	  //	  logger << LOG_INFO << "E[" << loop << "][" << type << "][" << i-1 << "] = " << E[loop][type][i-1] << endl;
	}
      }
      // Check what to do in SF case !!!
      // obtain quark form factor
      F[loop] = -E[loop][3][8]/4.0;
      logger << LOG_DEBUG << "F[" << loop << "] = " << F[loop] << endl;
    }
    
    reset_hel_amplitudes_H2_QCD_2loop(i_x);
    /*
    for (int loop=0; loop<3; loop++) {
      for (int h1=0; h1<2; h1++) {
	for (int h2=0; h2<2; h2++) {
	  for (int h3=0; h3<2; h3++) {
	    if (i_x == 0){M_dressed[h1][h2][h3][loop] = 0.;}
	    else {M_dressed2[h1][h2][h3][loop] = 0.;}
//	    cout << "before M_dressed[" << h1 << "][" << h2 << "][" << h3 << "][" << loop << "] = " << M_dressed[h1][h2][h3][loop] << endl;
	  }
	}
      }
    }
    */
    //    if (i_x == 0){M_dressed.resize(2, vector<vector<vector<double_complex> > > (2, vector<vector<double_complex> > (2, vector<double_complex> (3, 0.))));}
    //    else {M_dressed2.resize(2, vector<vector<vector<double_complex> > > (2, vector<vector<double_complex> > (2, vector<double_complex> (3, 0.))));}
    
    for (int i_a = 0; i_a < DR[i_x].size(); i_a++){
      calculate_H2_QCD_2loop_computeVVprimeHelAmplitudesDR(i_x, i_a);
      /*
      //      ppllll24_computeVVprimeHelAmplitudes(osi, DR[i_x][i_a].V1, DR[i_x][i_a].V2, q1, p_VVamp, E, M_dressed_tmp);
      if (i_x == 0){add_hel_amps(M_dressed,M_dressed_tmp);}
      else {add_hel_amps(M_dressed2,M_dressed_tmp);}
      //      add_hel_amps(M_dressed,M_dressed_tmp);
      */
      add_hel_amplitudes_H2_QCD_2loop(i_x);
    }
    
    for (int i_a = 0; i_a < SR[i_x].size(); i_a++){
      calculate_H2_QCD_2loop_computeVVprimeHelAmplitudesSR(i_x, i_a);
      /*
      //      ppllll24_computeVVprimeHelAmplitudesSR(osi, SR[i_x][i_a].resonant_V, SR[i_x][i_a].V1, SR[i_x][i_a].V2, q1, p_VVamp, F, M_dressed_tmp);
      if (i_x == 0){add_hel_amps(M_dressed,M_dressed_tmp);}
      else {add_hel_amps(M_dressed2,M_dressed_tmp);}
      //      add_hel_amps(M_dressed,M_dressed_tmp);    
      */
      add_hel_amplitudes_H2_QCD_2loop(i_x);
    }
    /*
    for (int loop=0; loop<3; loop++) {
      for (int h1=0; h1<2; h1++) {
	for (int h2=0; h2<2; h2++) {
	  for (int h3=0; h3<2; h3++) {
	    cout << "after M_dressed[" << h1 << "][" << h2 << "][" << h3 << "][" << loop << "] = " << M_dressed[h1][h2][h3][loop] << endl;
	  }
	}
      }
    }
    */
  }

  
  // square and add up helicity amplitudes
  osi->QT_A0=0;
  osi->QT_A1=0;
  osi->QT_A2=0;
  for (int h1=0; h1<2; h1++) {
    for (int h2=0; h2<2; h2++) {
      for (int h3=0; h3<2; h3++) {
        osi->QT_A0 += norm(M_dressed[h1][h2][h3][0])+norm(M_dressed2[h1][h2][h3][0]);
	//	cout << h1 << "," << h2 << "," << h3 << ": " << norm(M_dressed[h1][h2][h3][0])*pow(4*M_PI*osi->msi->alpha_e,4)/12 << endl;
        osi->QT_A1 += 2*real(M_dressed[h1][h2][h3][0]*conj(M_dressed[h1][h2][h3][1]))+2*real(M_dressed2[h1][h2][h3][0]*conj(M_dressed2[h1][h2][h3][1]));
        osi->QT_A2 += norm(M_dressed[h1][h2][h3][1])+norm(M_dressed2[h1][h2][h3][1]);
        osi->QT_A2 += 2*real(M_dressed[h1][h2][h3][0]*conj(M_dressed[h1][h2][h3][2]))+2*real(M_dressed2[h1][h2][h3][0]*conj(M_dressed2[h1][h2][h3][2]));
        if (h2==h3) {
          osi->QT_A0 -= 2.0*real(conj(M_dressed[h1][h2][h3][0])*M_dressed2[h1][h2][h3][0]);
          
          osi->QT_A1 -= 2.0*real(conj(M_dressed[h1][h2][h3][0])*M_dressed2[h1][h2][h3][1]);
          osi->QT_A1 -= 2.0*real(conj(M_dressed2[h1][h2][h3][0])*M_dressed[h1][h2][h3][1]);
          
          osi->QT_A2 -= 2.0*real(conj(M_dressed[h1][h2][h3][1])*M_dressed2[h1][h2][h3][1]);
          osi->QT_A2 -= 2.0*real(conj(M_dressed[h1][h2][h3][0])*M_dressed2[h1][h2][h3][2]);
          osi->QT_A2 -= 2.0*real(conj(M_dressed2[h1][h2][h3][0])*M_dressed[h1][h2][h3][2]);
        }
      }
    }
  }
  
  double alpha_e = osi->msi->alpha_e;

  osi->QT_A0 *= pow(4*M_PI*alpha_e,4);
  osi->QT_A0 /= (4*9); // averaging factor
  osi->QT_A0 *= 3; // color factor
  osi->QT_A0 *= sym_factor;

  osi->QT_A1 *= pow(4*M_PI*alpha_e,4);
  osi->QT_A1 /= (4*9); // averaging factor
  osi->QT_A1 *= 3; // color factor
  osi->QT_A1 *= sym_factor;

  osi->QT_A2 *= pow(4*M_PI*alpha_e,4);
  osi->QT_A2 /= (4*9); // averaging factor
  osi->QT_A2 *= 3; // color factor
  osi->QT_A2 *= sym_factor;

  osi->QT_A1 *= osi->alpha_S/2/M_PI;
  osi->QT_A2 *= pow(osi->alpha_S/2/M_PI,2);

  
  osi->QT_H1_delta_QCD = osi->QT_A1;
  osi->QT_H2_delta_QCD = osi->QT_A2;
  osi->QT_H1_delta_QCD /= 2;
  osi->QT_H2_delta_QCD /= 4;
  osi->QT_H1_delta_QCD /= (osi->alpha_S * inv2pi * osi->QT_A0);
  osi->QT_H2_delta_QCD /= (pow(osi->alpha_S * inv2pi, 2) * osi->QT_A0);   
  logger << LOG_DEBUG << "osi->QT_H1_delta_QCD = " << osi->QT_H1_delta_QCD << ", osi->QT_H2_delta_QCD = " << osi->QT_H2_delta_QCD << endl;
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


#ifdef OPENLOOPS

ppllll04_amplitude_OpenLoops_set::ppllll04_amplitude_OpenLoops_set(){
  Logger logger("ppllll04_amplitude_OpenLoops_set::ppllll04_amplitude_OpenLoops_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_amplitude_OpenLoops_set::~ppllll04_amplitude_OpenLoops_set(){
  Logger logger("ppllll04_amplitude_OpenLoops_set::~ppllll04_amplitude_OpenLoops_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


void ppllll04_amplitude_OpenLoops_set::initialization_process(){
  Logger logger("ppllll04_amplitude_OpenLoops_set::initialization_process");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  if (csi->type_contribution == "VT2" && csi->type_correction == "QCD"){
    initialization_H2_QCD_2loop();
  }
  amplitude_OpenLoops_set::initialization_process();
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

#endif


#ifdef RECOLA

ppllll04_amplitude_Recola_set::ppllll04_amplitude_Recola_set(){
  Logger logger("ppllll04_amplitude_Recola_set::ppllll04_amplitude_Recola_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "constructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}

ppllll04_amplitude_Recola_set::~ppllll04_amplitude_Recola_set(){
  Logger logger("ppllll04_amplitude_Recola_set::~ppllll04_amplitude_Recola_set ()");
  logger << LOG_DEBUG_VERBOSE << "started" << endl;

  logger << LOG_INFO << "destructor called..." << endl;

  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


void ppllll04_amplitude_Recola_set::initialization_process(){
  Logger logger("ppllll04_amplitude_Recola_set::initialization_process");
  logger << LOG_DEBUG_VERBOSE << "called" << endl;

  if (csi->type_contribution == "VT2" && csi->type_correction == "QCD"){
    initialization_H2_QCD_2loop();
  }
  amplitude_Recola_set::initialization_process();
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endl;
}


#endif
