#include "header.hpp"
#ifdef RECOLA

void amplitude_Recola_set::calculate_ME2_RA_QCD(){
  Logger logger("amplitude_Recola_set::calculate_ME2_RA_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << flush << endl;

  if (esi->cut_ps[0] > -1){
    double P_Recola[esi->p_parton[0].size()][4];
    set_phasespacepoint(P_Recola);

    if ((csi->type_contribution == "RA" ||
	 csi->type_contribution == "RRA") && osi->user->string_value[osi->user->string_map["model"]] != "Bornloop"){
      compute_process_rcl(1, P_Recola, "LO");
      get_squared_amplitude_rcl(1, csi->contribution_order_alpha_s, "LO", osi->value_ME2term[0]);
      logger << LOG_DEBUG_POINT << "Recola:     ME2_R  = " << setw(23) << setprecision(15) << osi->value_ME2term[0] << endl;

      if (!(osi->check_vanishing_ME2_end)){
	osi->flag_vanishing_ME2 = 0;
	compute_all_colour_correlations_rcl(1, P_Recola);
	for (int i_1 = 1; i_1 < csi->n_particle + 3; i_1++){
	  for (int i_2 = 1; i_2 < csi->n_particle + 3; i_2++){
	    //	    for (int i_2 = i_1 + 1; i_2 < csi->n_particle + 3; i_2++){
	    double A2cc;
	    get_colour_correlation_rcl(1, csi->contribution_order_alpha_s, i_1, i_2, A2cc);
	    logger << LOG_DEBUG << "Recola:     ccME2[" << i_1 << "][" << i_2 << "] = " << A2cc << endl;
	    if (abs(A2cc) > 1.e12 * abs(osi->value_ME2term[0])){
	      logger << LOG_DEBUG << "osi->value_ME2term[0] = 0. due to numerical cancellations!" << endl;
	      osi->flag_vanishing_ME2 = 1;
	      break;
	    }
	  }
	}
      }
    }
    else if (csi->type_contribution == "L2RA" ||
	       osi->user->string_value[osi->user->string_map["model"]] == "Bornloop"){
      compute_process_rcl(1, P_Recola, "NLO");
      get_squared_amplitude_rcl(1, csi->contribution_order_alpha_s, "NLO", osi->value_ME2term[0]);
      logger << LOG_DEBUG_POINT << "Recola:     ME2_R  = " << setw(23) << setprecision(15) << osi->value_ME2term[0] << endl;

      // This is probably not helpful for loop-induced processes (unstable ME'2 might be set to 0):
      /*
      osi->flag_vanishing_ME2 = 0;
      if (osi->value_ME2term[0] == 0.){
	osi->flag_vanishing_ME2 = 1;
	logger << LOG_DEBUG << "b_ME2 = 0. due to numerical cancellations!" << endl;
      }
      */
    }
    else {
      logger << LOG_FATAL << "Should not happen!" << endl;
    }
  }
  else {osi->value_ME2term[0] = 0.;}

  osi->calculate_dipole_QCD();

  logger << LOG_DEBUG_VERBOSE << "finished" << flush << endl;
}


void amplitude_Recola_set::calculate_dipole_Acc_QCD(int x_a){
  static Logger logger("amplitude_Recola_set::calculate_dipole_Acc_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << flush << endl;

  osi->RA_ME2_cc[x_a] = 0.;

  double P_dipole_Recola[esi->p_parton[0].size()][4];
  set_phasespacepoint_dipole(P_dipole_Recola, x_a);

  if ((csi->type_contribution == "RA" ||
       csi->type_contribution == "RRA") && osi->user->string_value[osi->user->string_map["model"]] != "Bornloop"){
    compute_process_rcl(x_a + 1, P_dipole_Recola, "LO");
    logger << LOG_DEBUG_VERBOSE << "compute_process_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << char(34) << "LO" << char(34) << ");" << endl;
    compute_colour_correlation_rcl(x_a + 1, P_dipole_Recola, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), osi->RA_ME2_cc[x_a]);
    logger << LOG_DEBUG_VERBOSE << "compute_colour_correlation_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << "RA_ME2_cc[" << x_a << "] -> " << osi->RA_ME2_cc[x_a] << ");" << endl;
    osi->RA_ME2_cc[x_a] = osi->RA_ME2_cc[x_a] * csi->dipole[x_a].colour_factor();
    logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_cc[" << x_a << "] = " << setw(23) << setprecision(15) << osi->RA_ME2_cc[x_a] << endl;
  }
  else if (csi->type_contribution == "L2RA" ||
	   osi->user->string_value[osi->user->string_map["model"]] == "Bornloop"){
    compute_process_rcl(x_a + 1, P_dipole_Recola, "NLO");
    logger << LOG_DEBUG_VERBOSE << "compute_process_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << char(34) << "NLO" << char(34) << ");" << endl;
    compute_colour_correlation_rcl(x_a + 1, P_dipole_Recola, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), "NLO", osi->RA_ME2_cc[x_a]);
    logger << LOG_DEBUG_VERBOSE << "compute_colour_correlation_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << char(34) << "NLO" << char(34) << "RA_ME2_cc[" << x_a << "] -> " << osi->RA_ME2_cc[x_a] << ");" << endl;
    logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_cc[" << x_a << "] = " << setw(23) << setprecision(15) << osi->RA_ME2_cc[x_a] << endl;
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << flush << endl;
}

void amplitude_Recola_set::calculate_dipole_Asc_QCD(int x_a, fourvector & Vtensor){
  static Logger logger("amplitude_Recola_set::calculate_dipole_Asc_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << flush << endl;

  double P_dipole_Recola[esi->p_parton[0].size()][4];
  set_phasespacepoint_dipole(P_dipole_Recola, x_a);

  double_complex MOM[4];
  MOM[0] = Vtensor.x0();
  MOM[1] = Vtensor.x1();
  MOM[2] = Vtensor.x2();
  MOM[3] = Vtensor.x3();
  stringstream temp_ss;
  for (int j = 0; j < 4; j++){
    temp_ss << setw(23) << setprecision(15) << MOM[j] << "   ";
  }

  if ((csi->type_contribution == "RA" ||
       csi->type_contribution == "RRA") && osi->user->string_value[osi->user->string_map["model"]] != "Bornloop"){
    compute_process_rcl(x_a + 1, P_dipole_Recola, "LO");
    logger << LOG_DEBUG_VERBOSE << "compute_process_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << char(34) << "LO" << char(34) << ");" << endl;
    compute_colour_correlation_rcl(x_a + 1, P_dipole_Recola, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), osi->RA_ME2_cc[x_a]);
    logger << LOG_DEBUG_VERBOSE << "compute_colour_correlation_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << "osi->RA_ME2_cc[x_a] -> " << osi->RA_ME2_cc[x_a] << ");" << endl;
    osi->RA_ME2_cc[x_a] = osi->RA_ME2_cc[x_a] * csi->dipole[x_a].colour_factor();
    logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_cc[" << x_a << "] = " << setw(23) << setprecision(15) << osi->RA_ME2_cc[x_a] << endl;

    compute_spin_colour_correlation_rcl(x_a + 1, P_dipole_Recola, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), MOM);
    logger << LOG_DEBUG_VERBOSE << "compute_spin_colour_correlation_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << "MOM);" << endl;
    get_spin_colour_correlation_rcl(x_a + 1, csi->contribution_order_alpha_s - 1, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), osi->RA_ME2_sc[x_a]);
    logger << LOG_DEBUG_VERBOSE << "get_spin_colour_correlation_rcl(" << x_a + 1 << ", " << csi->contribution_order_alpha_s - 1 << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << "RA_ME2_sc[" << x_a << "] -> " << osi->RA_ME2_sc[x_a] << ");" << endl;
    osi->RA_ME2_sc[x_a] = osi->RA_ME2_sc[x_a] * csi->dipole[x_a].colour_factor() / (-Vtensor.m2());
    logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_sc[" << x_a << "] = " << setw(23) << setprecision(15) << osi->RA_ME2_sc[x_a] << endl;

  }
  else if (csi->type_contribution == "L2RA" ||
	   osi->user->string_value[osi->user->string_map["model"]] == "Bornloop"){
    compute_process_rcl(x_a + 1, P_dipole_Recola, "NLO");
    logger << LOG_DEBUG_VERBOSE << "compute_process_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << char(34) << "NLO" << char(34) << ");" << endl;
    compute_colour_correlation_rcl(x_a + 1, P_dipole_Recola, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), "NLO", osi->RA_ME2_cc[x_a]);
    logger << LOG_DEBUG_VERBOSE << "compute_colour_correlation_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << char(34) << "NLO" << char(34) << ", " << "osi->RA_ME2_cc[x_a] -> " << osi->RA_ME2_cc[x_a] << ");" << endl;
    osi->RA_ME2_cc[x_a] = osi->RA_ME2_cc[x_a] * csi->dipole[x_a].colour_factor();
    logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_cc[" << x_a << "] = " << osi->RA_ME2_cc[x_a] << endl;

    compute_spin_colour_correlation_rcl(x_a + 1, P_dipole_Recola, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), MOM, "NLO");
    logger << LOG_DEBUG_VERBOSE << "compute_spin_colour_correlation_rcl(" << x_a + 1 << ", " << "P_dipole_Recola" << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << "MOM" << ", " << char(34) << "NLO" << char(34) << ");" << endl;
    get_spin_colour_correlation_rcl(x_a + 1, csi->contribution_order_alpha_s - 1, csi->dipole[x_a].no_A_emitter(), csi->dipole[x_a].no_A_spectator(), "NLO", osi->RA_ME2_sc[x_a]);
    logger << LOG_DEBUG_VERBOSE << "get_spin_colour_correlation_rcl(" << x_a + 1 << ", " << csi->contribution_order_alpha_s - 1 << ", " << csi->dipole[x_a].no_A_emitter() << ", " << csi->dipole[x_a].no_A_spectator() << ", " << "RA_ME2_sc[" << x_a << "] -> " << osi->RA_ME2_sc[x_a] << ");" << endl;
    osi->RA_ME2_sc[x_a] = osi->RA_ME2_sc[x_a] * csi->dipole[x_a].colour_factor() / (-Vtensor.m2());
    //    osi->RA_ME2_sc[x_a] = 0.; // temporary for OpenLoops-Recola validation
    logger << LOG_DEBUG_POINT << "Recola:        RA_ME2_sc[" << x_a << "] = " << osi->RA_ME2_sc[x_a] << endl;
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << flush << endl;
}

#endif
