#include "header.hpp"

void amplitude_set::calculate_ME2_born(){
  Logger logger("amplitude_set::calculate_ME2_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  logger << LOG_FATAL << "Basic amplitude_set function must be overwritten." << endline; exit(1);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void amplitude_set::calculate_ME2check_born(){
  Logger logger("amplitude_set::calculate_ME2check_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (osi->switch_output_comparison && process_id != -1){
    esi->phasespacepoint();
    if (esi->p_parton[0][0].x0() == 0.){generate_testpoint();}
    if (esi->p_parton[0][0].x0() != 0.){
      ofstream out_comparison;
      out_comparison.open(osi->filename_comparison.c_str(), ofstream::out | ofstream::app);

      esi->perform_event_selection();
      osi->set_testpoint_scale_complete();
      vector<int> cut_ps_acc_all(csi->n_ps, 0);
      swap(esi->cut_ps, cut_ps_acc_all);

      calculate_ME2_born();

      out_comparison << output_amplitude_born() << endline;
      //      out_comparison << osi->output_testpoint_born() << endline;
      out_comparison << endline;
      osi->set_testpoint_x_pdf();
      if (osi->switch_TSV){
	//	osi->calculate_pdf_LHAPDF_TSV();
	osi->calculate_pdf_LHAPDF_list_TSV();
	out_comparison << osi->output_testpoint_integrand() << endline;
	out_comparison << endline;
      }
  
      swap(esi->cut_ps, cut_ps_acc_all);
      out_comparison << osi->output_testpoint_kinematics() << endline;
      out_comparison << osi->output_scales() << endline;
      out_comparison.close();
    }
  }
  output_parameter("testpoint");

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


string amplitude_set::output_amplitude_born(){
  static Logger logger("amplitude_set::output_amplitude_born");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  stringstream out_comparison;
  stringstream scouplings_ss;
  scouplings_ss << "(alpha_s^" << csi->contribution_order_alpha_s << " alpha_e^" << csi->contribution_order_alpha_e << ")";
  string scouplings = scouplings_ss.str();
  double couplings = pow(osi->alpha_S, csi->contribution_order_alpha_s) * pow(msi->alpha_e, csi->contribution_order_alpha_e);

  out_comparison << "Born matrix elements at alpha_S(µR = " << setprecision(15) << osi->scale_ren << ") = " << setprecision(15) << osi->alpha_S << "" << endline << endline;
  //  out_comparison << "Born matrix elements (µR = " << osi->scale_ren << ", " << "alpha_S(µR) = " << setprecision(15) << osi->alpha_S << ")" << endline << endline;
  out_comparison << "ME2_born = " << setprecision(15) << showpoint << setw(23) << osi->value_ME2term[0] << endline << endline;
  out_comparison << "Born matrix elements devided by coupling constants: " << endline << endline;
  out_comparison << "ME2_born / " << scouplings << " = " << setprecision(15) << showpoint << setw(23) << osi->value_ME2term[0] / couplings << endline;

  logger << LOG_DEBUG_VERBOSE << "finished - return (string):" << endl << out_comparison.str() << endline_flush;
  return out_comparison.str();
}


