#include "header.hpp"

void amplitude_set::calculate_ME2_VA_QCD(){
  Logger logger("amplitude_set::calculate_ME2_VA_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  logger << LOG_FATAL << "Basic amplitude_set function must be overwritten." << endline; exit(1);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void amplitude_set::calculate_ME2_ioperator_VA_QCD(){
  Logger logger("amplitude_set::calculate_ME2_ioperator_VA_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  logger << LOG_FATAL << "Basic amplitude_set function must be overwritten." << endline; exit(1);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void amplitude_set::calculate_ME2check_VA_QCD(){
  static Logger logger("amplitude_set::calculate_ME2check_VA_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (osi->switch_output_comparison && process_id != -1){
    esi->phasespacepoint();
    if (esi->p_parton[0][0].x0() == 0.){generate_testpoint();}
    if (esi->p_parton[0][0].x0() != 0.){
      ofstream out_comparison;
      out_comparison.open(osi->filename_comparison.c_str(), ofstream::out | ofstream::app);

      esi->perform_event_selection();
      osi->set_testpoint_scale_complete();
      vector<int> cut_ps_acc_all(csi->n_ps, 0);
      swap(esi->cut_ps, cut_ps_acc_all);

      osi->VA_delta_flag = 1;
      osi->VA_b_ME2 = 0.;
      osi->VA_V_ME2 = 0.;
      osi->VA_X_ME2 = 0.;
      osi->VA_I_ME2 = 0.;
      osi->VA_X_ME2_CV.resize(osi->n_scales_CV, 0.);
      // Probably not implemented like this ...
      // 1 - Testpoint is calculated at basic fixed scale (prefactor * scale_ren).
      // 2 - Testpoint is calculated at output scale.
      if (osi->switch_output_comparison == 2){set_testpoint_alpha_S(osi->var_alpha_S_reference);}
      // With OpenLoops, MUNICH does not reset the value of alpha_S for different scales, but adds the corresponding relative factors.
      // For the test-point output, the value of alpha_S is set in OpenLoops here:
      //      osi->alpha_S = osi->alpha_S * osi->var_rel_alpha_S;
      //      osi->alpha_S = osi->alpha_S * pow(osi->var_rel_alpha_S, double(csi->contribution_order_alpha_s - 1) / csi->contribution_order_alpha_s);
      //      osi->alpha_S = osi->var_alpha_S_reference;
      //      ol_setparameter_double(OL_alpha_s, osi->alpha_S);
      osi->VA_DeltaUV = 0.;
      osi->VA_DeltaIR1 = 0.;
      osi->VA_DeltaIR2 = 0.;
      logger << LOG_DEBUG << "osi->VA_DeltaUV  = " << osi->VA_DeltaUV << endline_flush;
      logger << LOG_DEBUG << "osi->VA_DeltaIR1 = " << osi->VA_DeltaIR1 << endline_flush;
      logger << LOG_DEBUG << "osi->VA_DeltaIR2 = " << osi->VA_DeltaIR2 << endline_flush;

      set_mu_reg();
      set_mu_and_delta_VA();
      output_parameter("testpoint");
      calculate_ME2_VA_QCD();
      
      out_comparison << output_amplitude_VA_QCD();
      
      osi->set_testpoint_x_pdf();
      if (osi->switch_TSV){
	//	osi->calculate_pdf_LHAPDF_TSV();
	osi->calculate_pdf_LHAPDF_list_TSV();
	out_comparison << osi->output_testpoint_VA_integrand_TSV() << endline;
	out_comparison << endline;
	out_comparison << osi->output_testpoint_integrand() << endline;
	out_comparison << endline;
      }
 
      swap(esi->cut_ps, cut_ps_acc_all);
      out_comparison << osi->output_testpoint_kinematics() << endline;
      out_comparison << osi->output_scales() << endline;
      out_comparison.close();
    }
    else {
      output_parameter("testpoint");
    }

    osi->switch_output_comparison = 0;
  }
  else {
    output_parameter("testpoint");
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


string amplitude_set::output_amplitude_VA_QCD(){
  static Logger logger("amplitude_set::output_amplitude_VA_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  stringstream out_comparison;
  out_comparison << "Matrix elements (µreg = " << mu_reg << ", µR = " << osi->scale_ren << ", " << "alpha_S(µR) = " << setprecision(15) << osi->alpha_S << ")" << endline << endline;

  /*
    out_comparison << setw(12) << " ME2_born = " << setprecision(15) << setw(23) << osi->VA_b_ME2 << endline;
    out_comparison << setw(12) << "ME2_V+X+I = " << setprecision(15) << setw(23) << (osi->VA_V_ME2 + osi->VA_X_ME2 + osi->VA_I_ME2) << endline;
    out_comparison << setw(12) << "  ME2_V+X = " << setprecision(15) << setw(23) << (osi->VA_V_ME2 + osi->VA_X_ME2) << endline;
    out_comparison << endline;
    out_comparison << setw(12) << "    ME2_V = " << setprecision(15) << setw(23) << osi->VA_V_ME2 << endline;
    out_comparison << setw(12) << "    ME2_X = " << setprecision(15) << setw(23) << osi->VA_X_ME2 << endline;
    out_comparison << setw(12) << "    ME2_I = " << setprecision(15) << setw(23) << osi->VA_I_ME2 << endline;
    out_comparison << endline;
  */
  out_comparison << osi->output_testpoint_VA_result() << endline;
  
  double OL_I_ME2 = osi->VA_I_ME2;
  if (osi->switch_VI != 2){
    osi->VA_I_ME2 = 0.;
    set_testpoint_alpha_S(osi->var_alpha_S_reference); // Recola: !!!

    calculate_ME2_ioperator_VA_QCD();
    set_testpoint_rescale_I_ME2(); // Recola: !!!
    /*
    out_comparison << setw(12) << " SK_ME2_I = " << setprecision(15) << setw(23) << osi->VA_I_ME2 << endline;
    out_comparison << endline;
    for (int i_a = 0; i_a < (*osi->VA_ioperator).size(); i_a++){
      for (int j_a = 0; j_a < (*osi->VA_ioperator)[i_a].size(); j_a++){
	out_comparison << left << setw(26) << (*osi->VA_ioperator)[i_a][j_a].name() << ": " << "I_ME2_cf[i_a = " << setw(2) << right << i_a << "][j_a = " << setw(2) << right << j_a << "] = " << setw(23) << setprecision(15) << osi->VA_I_ME2_cf[i_a][j_a] << "   ME2_cf[" << setw(2) << right << i_a << "][" << setw(2) << right << j_a << "] = " << setw(23) << setprecision(15) << osi->VA_ME2_cf[i_a][j_a] << endline;
      }
    }
    out_comparison << endline;
    out_comparison << endline;
    */
    out_comparison << osi->output_testpoint_VA_ioperator() << endline;
    
    osi->VA_I_ME2 = OL_I_ME2;
    //    osi->output_testpoint_VA_ioperator(out_comparison);
  }

  string s_Delta;
  double i_Delta;
  
  out_comparison << "Numerical check of (UV and IR) finiteness: " << endline << endline;

  if (allowed_Delta_IRneqUV){
    s_Delta = "Delta_UV";
    for (int i = 0; i < 3; i++){
      i_Delta = (double(i) - 1.) * 1.;
      osi->VA_DeltaUV = i_Delta;
      set_mu_and_delta_VA();
      calculate_ME2_VA_QCD();
      out_comparison << osi->output_testpoint_VA_Delta(i, i_Delta, s_Delta) << endline;
    }
    osi->VA_DeltaUV = 0.;
    out_comparison << endline;

    s_Delta = "Delta_IR_1";
    for (int i = 0; i < 3; i++){
      i_Delta = (double(i) - 1.) * 1.;
      osi->VA_DeltaIR1 = i_Delta;
      set_mu_and_delta_VA();
      calculate_ME2_VA_QCD();
      out_comparison << osi->output_testpoint_VA_Delta(i, i_Delta, s_Delta) << endline;
    }
    osi->VA_DeltaIR1 = 0.;
    out_comparison << endline;
  }

  s_Delta = "Delta_IR_2";
  for (int i = 0; i < 3; i++){
    i_Delta = (double(i) - 1.) * 1.;
    osi->VA_DeltaIR2 = i_Delta;
    set_mu_and_delta_VA();
    calculate_ME2_VA_QCD();
    out_comparison << osi->output_testpoint_VA_Delta(i, i_Delta, s_Delta) << endline;
  }
  osi->VA_DeltaIR2 = 0.;
  out_comparison << endline;

  s_Delta = "Delta_UV = Delta_IR_1";
  for (int i = 0; i < 3; i++){
    i_Delta = (double(i) - 1.) * 1.;
    osi->VA_DeltaUV = i_Delta;
    osi->VA_DeltaIR1 = i_Delta;
    set_mu_and_delta_VA();
    calculate_ME2_VA_QCD();
    out_comparison << osi->output_testpoint_VA_Delta(i, i_Delta, s_Delta) << endline;
  }

  osi->VA_DeltaUV = 0.;
  osi->VA_DeltaIR1 = 0.;
  out_comparison << endline;
  out_comparison << endline;
  set_mu_and_delta_VA();
  calculate_ME2_VA_QCD();

  osi->VA_delta_flag = 0;
 
  if (osi->switch_output_comparison == 2){set_testpoint_alpha_S(osi->alpha_S);}
  // With OpenLoops, MUNICH does not reset the value of alpha_S for different scales, but adds the corresponding relative factors.
  // For the test-point output, the value of alpha_S is set in OpenLoops and reset to its standard value here:
  //      osi->alpha_S = osi->alpha_S / osi->var_rel_alpha_S;
  ///      osi->alpha_S = osi->alpha_S / pow(osi->var_rel_alpha_S, double(csi->contribution_order_alpha_s - 1) / csi->contribution_order_alpha_s);
  // Check if comments still make sense !!!
  
  logger << LOG_DEBUG_VERBOSE << "finished - return (string):" << endl << out_comparison.str() << endline_flush;
  return out_comparison.str();
}
