#include "header.hpp"

void amplitude_set::calculate_ME2_VT2_QCD(){
  Logger logger("amplitude_set::calculate_ME2_VT2_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  logger << LOG_FATAL << "Basic amplitude_set function must be overwritten." << endline; exit(1);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void amplitude_set::calculate_ME2check_VT2_QCD(){
  static Logger logger("amplitude_set::calculate_ME2check_VT2_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  if (osi->switch_output_comparison && process_id != -1){
    esi->phasespacepoint();
    if (esi->p_parton[0][0].x0() == 0.){generate_testpoint();}
    if (esi->p_parton[0][0].x0() != 0.){
      ofstream out_comparison;
      out_comparison.open(osi->filename_comparison.c_str(), ofstream::out | ofstream::app);

      esi->perform_event_selection();
      osi->set_testpoint_scale_complete();
      vector<int> cut_ps_acc_all(csi->n_ps, 0);
      swap(esi->cut_ps, cut_ps_acc_all);

      calculate_ME2_VT2_QCD();
      osi->psi->set_xz_coll_pdf();
      osi->set_testpoint_g_z_coll();
      osi->set_testpoint_psp();

      osi->calculate_pdf_LHAPDF_list_CV();
      if (osi->switch_TSV){osi->calculate_pdf_LHAPDF_list_TSV();}
      osi->determine_integrand_CX_ncollinear_VT2();

      out_comparison << output_amplitude_VT2_QCD();
      out_comparison << osi->output_ME2_VT2_QCD();

      swap(esi->cut_ps, cut_ps_acc_all);
      out_comparison << osi->output_testpoint_kinematics() << endline;
      out_comparison << osi->output_scales() << endline;
      out_comparison.close();
    }
  }
  output_parameter("testpoint");

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


#define Fcw left << setw(17)
#define FldXV setprecision(15) << setw(23) << left

string amplitude_set::output_amplitude_VT2_QCD(){
  static Logger logger("amplitude_set::output_amplitude_VT2_QCD");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  stringstream out_comparison;
  out_comparison << "Matrix elements (µreg = " << mu_reg << ", µR = " << mu_reg << ", " << "alpha_S(µR = " << osi->scale_ren << ") = " << setprecision(15) << osi->alpha_S << ")" << endline;
  out_comparison << endline;
  out_comparison << Fcw << "ME2_born" << " = " << FldXV << osi->VA_b_ME2 << endline;
  out_comparison << Fcw << "ME2_V+X" << " = " << FldXV << (osi->VA_V_ME2 + osi->VA_X_ME2) << endline;
  out_comparison << endline;
  out_comparison << Fcw << "ME2_V" << " = " << FldXV << osi->VA_V_ME2 << endline;
  out_comparison << Fcw << "ME2_X" << " = " << FldXV << osi->VA_X_ME2 << endline;
  out_comparison << endline;
  out_comparison << Fcw << "H1_delta_QCD" << " = " << FldXV << osi->QT_H1_delta_QCD << endline;
  out_comparison << Fcw << "H2_delta_QCD" << " = " << FldXV << osi->QT_H2_delta_QCD << endline;
  out_comparison << endline;
  
  logger << LOG_DEBUG_VERBOSE << "finished - return (string):" << endl << out_comparison.str() << endline_flush;
  return out_comparison.str();
}
