#include "header.hpp"

void observable_set::determine_collinear_QEW(){
  Logger logger("observable_set::determine_collinear_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  logger << LOG_DEBUG << "Collinear-dipole determination" << endline << endline_flush;
  logger.newLine(LOG_DEBUG);

  vector<string> pa_name(csi->type_parton[0].size(), "");
  // Gluons should actually not be removed here in general !!!
  // -> Any IS particle can be spectator for photon emitters !!!
  if (msi->charge_pdg[csi->type_parton[0][1]] != 0. || csi->type_parton[0][1] == 22){pa_name[1] = "a";}
  if (msi->charge_pdg[csi->type_parton[0][2]] != 0. || csi->type_parton[0][2] == 22){pa_name[2] = "b";}
  //  pa_name[1] = "a";
  //  pa_name[2] = "b";
  for (int i_p = 3; i_p < pa_name.size(); i_p++){if (msi->charge_pdg[csi->type_parton[0][i_p]] != 0. || csi->type_parton[0][i_p] == 22){pa_name[i_p] = char(105 + i_p - 3);}}

  int temp_type_correction = 2;

  for (int temp_no_emitter = 1; temp_no_emitter < 3; temp_no_emitter++){
    //  for (int temp_no_emitter = 1; temp_no_emitter < csi->type_parton[0].size(); temp_no_emitter++){
    // Could be an option to allow for final-state emitter (with initial-state spectator) remnants !!!
    // At present, those are included in initial-final contributions !!!
    if (pa_name[temp_no_emitter] == ""){continue;}
    string temp_name;
    vector<string> temp_all_name(csi->combination_pdf.size());
    int temp_type;
    vector<int> temp_in_collinear(3, 0);
    vector<vector<int> > temp_pdf_new;
    double temp_charge_factor = 1.;
    double temp_charge_factor_fi = 1.;
    for (int i_t = 0; i_t < 2; i_t++){
      temp_in_collinear[temp_no_emitter] = 1;

      temp_pdf_new = csi->combination_pdf;

      if (csi->combination_pdf[0][temp_no_emitter] == 7){
	if (i_t == 0){
	  temp_type = 0; // hard process with a from a -> a a splitting (not in QED ???)
	  temp_in_collinear[0] = 1;
	  temp_in_collinear[temp_no_emitter] = 0;
	  temp_name = "C_" + pa_name[temp_no_emitter] + "^{aa}";
	  for (int i_x = 0; i_x < csi->combination_pdf.size(); i_x++){temp_all_name[i_x] = "C_" + pa_name[temp_no_emitter] + "^{aa}";}
	}
	else {
	  temp_type = 2; // hard process with a from q -> q a splitting
	  temp_in_collinear[0] = 0;
	  temp_name = "C_" + pa_name[temp_no_emitter] + "^{qa}";
	  for (int i_x = 0; i_x < csi->combination_pdf.size(); i_x++){temp_pdf_new[i_x][temp_no_emitter] = 10;}
	  for (int i_x = 0; i_x < csi->combination_pdf.size(); i_x++){temp_all_name[i_x] = "C_" + pa_name[temp_no_emitter] + "^{qa}";}
	}
      }
      else if (csi->combination_pdf[0][temp_no_emitter] != 7){
	if (i_t == 0){
	  temp_type = 3; // hard process with q/qx from a -> q qx splitting
	  temp_in_collinear[0] = 0;
	  temp_name = "C_" + pa_name[temp_no_emitter] + "^{a" + csi->name_particle[csi->combination_pdf[0][temp_no_emitter]] + "}";
	  for (int i_x = 0; i_x < csi->combination_pdf.size(); i_x++){temp_pdf_new[i_x][temp_no_emitter] = 7;}
	  for (int i_x = 0; i_x < csi->combination_pdf.size(); i_x++){temp_all_name[i_x] = "C_" + pa_name[temp_no_emitter] + "^{a" + csi->name_particle[csi->combination_pdf[i_x][temp_no_emitter]] + "}";}
	}
	else {
	  temp_type = 1; // hard process with q from q -> q a splitting
	  temp_in_collinear[0] = 1;
	  temp_name = "C_" + pa_name[temp_no_emitter] + "^{" + csi->name_particle[csi->combination_pdf[0][temp_no_emitter]] + csi->name_particle[csi->combination_pdf[0][temp_no_emitter]] + "}";
	  for (int i_x = 0; i_x < csi->combination_pdf.size(); i_x++){temp_all_name[i_x] = "C_" + pa_name[temp_no_emitter] + "^{" + csi->name_particle[csi->combination_pdf[i_x][temp_no_emitter]] + csi->name_particle[csi->combination_pdf[i_x][temp_no_emitter]] + "}";}
	}
      }

      // Check if better shifted to PDF selection routine !!!
      if ((LHAPDF::hasPhoton() == 0) && (temp_type == 0 || temp_type == 3 || csi->type_parton[0][temp_no_emitter % 2 + 1] == 22)){
	logger << LOG_DEBUG << "No photon pdf available in pdf set " << LHAPDFname << endline_flush;
	continue;
      }

      int flag = (*CA_collinear).size();
      for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){if (temp_no_emitter == (*CA_collinear)[i_c][0].no_emitter() && temp_type == (*CA_collinear)[i_c][0].type()){flag = i_c; break;}}
      if (flag == (*CA_collinear).size()){(*CA_collinear).push_back(vector<collinear_set> ());}
      for (int temp_no_spectator = 0; temp_no_spectator < csi->type_parton[0].size(); temp_no_spectator++){
	if ((temp_no_spectator > 0 && pa_name[temp_no_spectator] == "") || temp_no_spectator == temp_no_emitter){continue;}
	vector<int> temp_pair(2);
	if (temp_no_emitter < temp_no_spectator){
	  temp_pair[0] = temp_no_emitter;
	  temp_pair[1] = temp_no_spectator;
	}
	else {
	  temp_pair[0] = temp_no_spectator;
	  temp_pair[1] = temp_no_emitter;
	}

	int temp_massive = -1;
	if (csi->mass_parton[0][temp_no_emitter] == 0. && csi->mass_parton[0][temp_no_spectator] == 0.){temp_massive = 0;}
	else if (csi->mass_parton[0][temp_no_emitter] != 0. && csi->mass_parton[0][temp_no_spectator] == 0.){temp_massive = 1;}
	else if (csi->mass_parton[0][temp_no_emitter] == 0. && csi->mass_parton[0][temp_no_spectator] != 0.){temp_massive = 2;}
	else if (csi->mass_parton[0][temp_no_emitter] != 0. && csi->mass_parton[0][temp_no_spectator] != 0.){temp_massive = 3;}
	else {logger << LOG_FATAL << "Should not happen!" << endline_flush; exit(1);}

	// Clean up once all cases are fully validated !!!
	
	//  temp_charge_factor_fi needs to be divided by corresponding CA_Q2f[i_c][0] as this factor is provided with the PDFs
	//  The critical case of q -> a (q) splittings with different CA_Q2f[i_c][i_x] does not contribute due to temp_charge_factor_fi = 0. for charged fermions !!!
	//  temp_charge_factor_fi = temp_charge_factor_fi /CA_Q2f[i_c][0];
	if (temp_no_spectator == 0){
	  temp_charge_factor = 1.;
	  //	  temp_charge_factor = pow(msi->charge_pdg[abs(csi->type_parton[0][temp_no_emitter])], 2);
	  temp_charge_factor_fi = 0.;
	}
	else if (csi->type_parton[0][temp_no_emitter] != 22 && csi->type_parton[0][temp_no_spectator] != 22){
	  temp_charge_factor = msi->charge_pdg[csi->type_parton[0][temp_no_spectator]] / msi->charge_pdg[csi->type_parton[0][temp_no_emitter]]; // (Q_ij Q_k) / Q²_ij = Q_k / Q_ij
	  if ((temp_no_emitter > 2 && temp_no_spectator < 3) || (temp_no_emitter < 3 && temp_no_spectator > 2)){temp_charge_factor *= -1;}
	  temp_charge_factor_fi = 1. / temp_charge_factor;
	  // To match Luca's implementation; this factor could be missing in Marek's summary of the subtraction method !!!
	  temp_charge_factor_fi *= msi->charge2_pdg[csi->type_parton[0][temp_no_spectator]];
	  // in that case, no charge_factor_fi would be needed at all !!! "spectator" is actually the emitter in the final-initial contribution
	  logger << LOG_DEBUG << "20210203 changed" << endline_flush;
	  logger << LOG_DEBUG << "msi->charge_pdg[csi->type_parton[0][" << temp_no_spectator << "]] = " << msi->charge_pdg[csi->type_parton[0][temp_no_spectator]] << endline_flush;
	  logger << LOG_DEBUG << "msi->charge_pdg[csi->type_parton[0][" << temp_no_emitter << "]] = " << msi->charge_pdg[csi->type_parton[0][temp_no_emitter]] << endline_flush;
	  logger << LOG_DEBUG << "temp_charge_factor = " << temp_charge_factor << endline_flush;
	  logger << LOG_DEBUG << "temp_charge_factor_fi = " << temp_charge_factor_fi << endline_flush;
	  /*
	  // 20210203 changed
	  temp_charge_factor = msi->charge_pdg[abs(csi->type_parton[0][temp_no_spectator])] / msi->charge_pdg[abs(csi->type_parton[0][temp_no_emitter])]; // (Q_ij Q_k) / Q²_ij
	  // seems to be the other way round, which looks also more reasonable... // before: Q²_ij / (Q_ij Q_k)
	  //	  temp_charge_factor = msi->charge_pdg[abs(csi->type_parton[0][temp_no_emitter])] * msi->charge_pdg[abs(csi->type_parton[0][temp_no_spectator])];
	  if ((temp_no_emitter > 2 && csi->type_parton[0][temp_no_emitter] > 0) || (temp_no_emitter < 3 && csi->type_parton[0][temp_no_emitter] < 0)){temp_charge_factor = -temp_charge_factor;}
	  // * (-1) for outgoing quark / incoming antiquark as emitter (only the latter makes sense here !!!)
	  if ((temp_no_spectator > 2 && csi->type_parton[0][temp_no_spectator] > 0) || (temp_no_spectator < 3 && csi->type_parton[0][temp_no_spectator] < 0)){temp_charge_factor = -temp_charge_factor;}
	  // * (-1) for outgoing quark / incoming antiquark as spectator
	  // Remainder of final-initial dipoles:
	  if (temp_no_spectator > 2){ // only contributing for FS emitter
	    temp_charge_factor_fi = msi->charge_pdg[abs(csi->type_parton[0][temp_no_emitter])] / msi->charge_pdg[abs(csi->type_parton[0][temp_no_spectator])]; // (Q_ij Q_k) / Q²_k
	    if ((temp_no_emitter > 2 && csi->type_parton[0][temp_no_emitter] > 0) || (temp_no_emitter < 3 && csi->type_parton[0][temp_no_emitter] < 0)){temp_charge_factor_fi = -temp_charge_factor_fi;}
	    if ((temp_no_spectator > 2 && csi->type_parton[0][temp_no_spectator] > 0) || (temp_no_spectator < 3 && csi->type_parton[0][temp_no_spectator] < 0)){temp_charge_factor_fi = -temp_charge_factor_fi;}
	  }
	  */
	}
	
	// short (but equivalent) version:
	else if (temp_no_spectator < 3){
	  if (csi->type_parton[0][temp_no_emitter] == 22){temp_charge_factor = -1.;} // kappa_ij,k = -1 for the other initial-state particle (0 for all final-state particles)
	  else {temp_charge_factor = 0.;} // not contributing because of photon (csi->type_parton[0][temp_no_spectator] == 22) charge Q_ij = 0 in (Q_ij Q_k) / Q²_k (has to be Q²_ij ???)
	  temp_charge_factor_fi = 0.; // only contributing for FS emitter
	}
	else { // case: (final-state spectator and emitter and/or spectator are photons)
	  temp_charge_factor = 0.; // not contributing because of kappa_ij,k = 0 for all final-state particles (em == 22 && sp != 22) or photon charge Q_ij = 0 (em != 22 && sp == 22) in (Q_ij Q_k) / Q²_k
	  if (csi->type_parton[0][temp_no_spectator] == 22){ // contribution for off-shell or unresolved photon final-state emitter, no contribution for hard on-shell photons
	    // not sure if this works in general !!!
	    if (temp_no_spectator > 2 + csi->n_particle_born){temp_charge_factor_fi = -0.5;} // unresolved photon final-state emitter (not present at LO), kappa_ij,k = -0.5 for each initial-state spectator
	    else {
	      if (csi->type_hadron[temp_no_spectator] == 22){temp_charge_factor_fi = 0.;} // hard on-shell photon does not undergo gamma->ffx splittings
	      else {temp_charge_factor_fi = -0.5;} // hard off-shell photon undergoes gamma->ffx splittings, kappa_ij,k = -0.5 for each initial-state spectator
	    }
	  }
	  else {temp_charge_factor_fi = 0.;} // not contributing because of photon (csi->type_parton[0][temp_no_spectator] == 22) charge Q_ij = 0 in (Q_ij Q_k) / Q²_k
	}

	/*
	// long version:
	else if (csi->type_parton[0][temp_no_emitter] == 22 && csi->type_parton[0][temp_no_spectator] != 22){
	  if (temp_no_spectator < 3){
	    temp_charge_factor = -1.; // kappa_ij,k = -1 for the other initial-state particle (0 for all final-state particles)
	    temp_charge_factor_fi = 0.; // only contributing for FS emitter
	  }
	  else {
	    temp_charge_factor = 0.; // kappa_ij,k = 0 for all final-state particles (-1 for other initial-state particle)
	    temp_charge_factor_fi = 0.; // not contributing because of photon charge Q_ij = 0 in (Q_ij Q_k) / Q²_k
	  }
	}
	else if (csi->type_parton[0][temp_no_emitter] != 22 && csi->type_parton[0][temp_no_spectator] == 22){
	  if (temp_no_spectator < 3){
	    temp_charge_factor = 0.; // not contributing because of photon charge Q_ij = 0 in (Q_ij Q_k) / Q²_k
	    temp_charge_factor_fi = 0.; // only contributing for FS emitter
	  }
	  else {
	    temp_charge_factor = 0; // not contributing because of photon charge Q_ij = 0 in (Q_ij Q_k) / Q²_k
	    if (csi->type_parton[0][temp_no_spectator] == 22){ // contribution for off-shell or unresolved photon final-state emitter, no contribution for hard on-shell photons
	      if (temp_no_spectator > csi->type_hadron.size()){
		temp_charge_factor_fi = -0.5; // unresolved photon final-state emitter (not present at LO), kappa_ij,k = -0.5 for each initial-state spectator
	      }
	      else {
		if (csi->type_hadron[temp_no_spectator] == 22){temp_charge_factor_fi = 0.;} // hard on-shell photon does not undergo gamma->ffx splittings
		else {temp_charge_factor_fi = -0.5;} // hard of-shell photon undergoes gamma->ffx splittings, kappa_ij,k = -0.5 for each initial-state spectator
	      }
	    }
	  }
	}
	else if (csi->type_parton[0][temp_no_emitter] == 22 && csi->type_parton[0][temp_no_spectator] == 22){
	  if (temp_no_spectator < 3){
	    temp_charge_factor = -1.; // kappa_ij,k = -1 for the other initial-state particle (0 for all final-state particles)
	    temp_charge_factor_fi = 0.; // only contributing for FS emitter
	  }
	  else {
	    temp_charge_factor = 0.; // kappa_ij,k = 0 for all final-state particles (-1 for other initial-state particle)
	    if (csi->type_parton[0][temp_no_spectator] == 22){ // contribution for off-shell or unresolved photon final-state emitter, no contribution for hard on-shell photons
	      if (temp_no_spectator > csi->type_hadron.size()){
		temp_charge_factor_fi = -0.5; // unresolved photon final-state emitter (not present at LO), kappa_ij,k = -0.5 for each initial-state spectator
	      }
	      else {
		if (csi->type_hadron[temp_no_spectator] == 22){temp_charge_factor_fi = 0.;} // hard on-shell photon does not undergo gamma->ffx splittings
		else {temp_charge_factor_fi = -0.5;} // hard of-shell photon undergoes gamma->ffx splittings, kappa_ij,k = -0.5 for each initial-state spectator
	      }
	    }
	    else {temp_charge_factor_fi = 0.;} // no contribution for charged final-state emitter
	  }
	  //	  temp_charge_factor = 1.;
	  // charge factor should depend on the respective charge of the splitting quarks/antiquarks - not implemented in a first version in the pdf routines!!!
	  ///	  temp_charge_factor = msi->charge_pdg[abs(csi->type_parton[0][dipole_candidate[i_a].no_R_emitter_1()])] * msi->charge_pdg[abs(csi->type_parton[0][dipole_candidate[i_a].no_R_emitter_2()])];
	  ///	  if ((dipole_candidate[i_a].no_R_emitter_1() > 2 && csi->type_parton[0][dipole_candidate[i_a].no_R_emitter_1()] > 0) || (dipole_candidate[i_a].no_R_emitter_1() < 3 && csi->type_parton[0][dipole_candidate[i_a].no_R_emitter_1()] < 0)){temp_charge_factor = -temp_charge_factor;}
	  ///	  if ((dipole_candidate[i_a].no_R_emitter_2() > 2 && csi->type_parton[0][dipole_candidate[i_a].no_R_emitter_2()] > 0) || (dipole_candidate[i_a].no_R_emitter_2() < 3 && csi->type_parton[0][dipole_candidate[i_a].no_R_emitter_2()] < 0)){temp_charge_factor = -temp_charge_factor;}
        }
	else {temp_charge_factor = 0.;}
	// case of outgoing photons not correctly implemented !!!
	*/

	logger << LOG_DEBUG << "temp_charge_factor = " << temp_charge_factor << endline_flush;
	logger << LOG_DEBUG << "temp_charge_factor_fi = " << temp_charge_factor_fi << endline_flush;


	vector<string> temp_all_name_spectator = temp_all_name;
	for (int i_x = 0; i_x < temp_all_name_spectator.size(); i_x++){temp_all_name_spectator[i_x] = temp_all_name_spectator[i_x] + "(" + pa_name[temp_no_spectator] + ")";}
	string temp_name_spectator = temp_name + "(" + pa_name[temp_no_spectator] + ")";

	(*CA_collinear)[flag].push_back(collinear_set(temp_name_spectator, temp_all_name_spectator, temp_type, temp_in_collinear, csi->no_process_parton[0], csi->type_parton[0], temp_pdf_new, temp_charge_factor, temp_charge_factor_fi, temp_no_emitter, temp_no_spectator, temp_pair, temp_type_correction, temp_massive));
      }
    }
  }

  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << "Before selection of contributing collinear 'dipoles':" << endline_flush;
  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << output_collinear() << endline_flush;
  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG  << "Before filling CA_combination_pdf for collinear 'dipoles'" << endline_flush;
  logger.newLine(LOG_DEBUG);

  fill_CA_combination_pdf();
  // old position !!!
  //  determine_CA_list_combination_pdf();

  logger << LOG_DEBUG  << "After filling CA_combination_pdf for collinear 'dipoles'" << endline_flush;
  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << "Before filling CA_Q2f for collinear 'dipoles':" << endline_flush;
  logger.newLine(LOG_DEBUG);
  // CA_Q2f  is used only when combining with pdfs and provides the correct charge factors (up- or down-type):
  // type = 0:   a -> a (a)   CA_Q2f[i_c][i_i][i_x] = 1.
  // type = 1:   q -> q (a)   CA_Q2f[i_c][i_i][i_x] = msi->charge2_pdg[abs((*CA_collinear)[i_c][0].type_parton()[(*CA_collinear)[i_c][0].no_emitter()])]
  // type = 2:   q -> a (q)   CA_Q2f[i_c][i_i][i_x] = msi->charge2_pdg[abs(CA_combination_pdf[i_c][i_i][i_x][(*CA_collinear)[i_c][0].no_emitter()])]
  // type = 3:   a -> q (a)   CA_Q2f[i_c][i_i][i_x] = msi->charge2_pdg[abs((*CA_collinear)[i_c][0].type_parton()[(*CA_collinear)[i_c][0].no_emitter()])]
  // This factor is always extracted from the corresponding functions for fermions (Kbar, Kt, gamma, K, etc.) and added when combining with PDFs (different QED charges in q-> a (q) splittings).
  CA_Q2f.resize((*CA_collinear).size());
  for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
    CA_Q2f[i_c].resize(CA_combination_pdf[i_c].size());
    for (int i_i = 0; i_i < CA_combination_pdf[i_c].size(); i_i++){
      for (int i_x = 0; i_x < CA_combination_pdf[i_c][i_i].size(); i_x++){
	//(*CA_collinear)[i_c][i_x] -> (*CA_collinear)[i_c][i_i] -> (*CA_collinear)[i_c][0] (0 component should always exist - and value are identical for all sub-dipoles.)
	if ((*CA_collinear)[i_c][0].type_parton()[(*CA_collinear)[i_c][0].no_emitter()] == 22){  // emitter is a photon
	  if ((*CA_collinear)[i_c][0].type() == 0){  // hard process with a from a -> a a splitting
	    CA_Q2f[i_c][i_i].push_back(1.);
	    logger << LOG_DEBUG << "type = " << (*CA_collinear)[i_c][0].type() << "   CA_Q2f[" << i_c << "][" << i_i << "][" << CA_Q2f[i_c][i_i].size() - 1 << "] = " << CA_Q2f[i_c][i_i][CA_Q2f[i_c][i_i].size() - 1] << endline_flush;
	  }
	  else if ((*CA_collinear)[i_c][0].type() == 2){  // hard process with a from q -> q a splitting
	    CA_Q2f[i_c][i_i].push_back(msi->charge2_pdg[abs(CA_combination_pdf[i_c][i_i][i_x][(*CA_collinear)[i_c][0].no_emitter()])]);
	    //	    logger << LOG_DEBUG << "type = " << (*CA_collinear)[i_c][0].type() << "   CA_Q2f[" << i_c << "][" << i_i << "][" << CA_Q2f[i_c][i_i].size() - 1 << "] = " << CA_Q2f[i_c][i_i][CA_Q2f[i_c][i_i].size() - 1] << endline_flush;
	  }
	  else {logger << LOG_WARN << "Wrong splitting: may not happen!" << endline_flush; exit(1);}
	}
	else { // emitter and spectator are charged particles
	  CA_Q2f[i_c][i_i].push_back(msi->charge2_pdg[abs((*CA_collinear)[i_c][0].type_parton()[(*CA_collinear)[i_c][0].no_emitter()])]);
	  logger << LOG_DEBUG << "type = " << (*CA_collinear)[i_c][0].type() << "   CA_Q2f[" << i_c << "][" << i_i << "][" << CA_Q2f[i_c][i_i].size() - 1 << "] = " << CA_Q2f[i_c][i_i][CA_Q2f[i_c][i_i].size() - 1] << endline_flush;
	}
      }
    }
  }
  
  logger << LOG_DEBUG << "After filling CA_Q2f for collinear 'dipoles'" << endline_flush;
  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << "Before removal of CA_collinear contributions that vanish due to charge_factor" << endline_flush;
  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << "(*CA_collinear).size() = " << (*CA_collinear).size() << endline_flush;
  logger << LOG_DEBUG << "CA_Q2f.size() = " << CA_Q2f.size() << endline_flush;
  for (int i_c = (*CA_collinear).size() - 1; i_c >= 0; i_c--){
    for (int j_c = (*CA_collinear)[i_c].size() - 1; j_c >= 0; j_c--){
      if ((*CA_collinear)[i_c][j_c].charge_factor() == 0. && (*CA_collinear)[i_c][j_c].charge_factor_fi() == 0.){    // Only erase if also charge_factor_fi == 0 !!!
	(*CA_collinear)[i_c].erase((*CA_collinear)[i_c].begin() + j_c);
      }
      else if ((*CA_collinear)[i_c][j_c].charge_factor() == 0. && (*CA_collinear)[i_c][j_c].charge_factor_fi() != 0.){
	logger << LOG_DEBUG << "This collinear term receives remainder contributions from final-initial dipoles only (switched off previously due to charge_factor = 0)." << endline_flush;
	logger << LOG_DEBUG << "Make sure there are no unwanted contributions now (Should be controlled via CA_ME2[i_c][j_c] !!!" << endline_flush;
      }
    }
    if ((*CA_collinear)[i_c].size() == 0){
      (*CA_collinear).erase((*CA_collinear).begin() + i_c);
      CA_Q2f.erase(CA_Q2f.begin() + i_c);
    }
  }

  
  // position after removal of vanishing dipoles !!!
  determine_CA_list_combination_pdf();

  
  // QT_Q2f nd CA_C2f should be made a single variable later !!!
  /*
  QT_Q2f = CA_Q2f;
  if (list_combination_pdf.size() == (*CA_collinear).size() + 1){QT_Q2f.push_back(QT_Q2f[0]);}
  */
  // fill endpoint component first (needs to be checked anyway !!!)
  QT_Q2f.resize((*CA_collinear).size() + 1);
  for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){QT_Q2f[i_c + 1] = CA_Q2f[i_c];}
  if ((*CA_collinear).size() > 0){QT_Q2f[0] = CA_Q2f[0];}


  
  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << "After removal of CA_collinear contributions that vanish due to charge_factor" << endline_flush;
  logger.newLine(LOG_DEBUG);

  determination_collinear_splitting();

  logger << LOG_DEBUG << "After determination of collinear_splitting" << endline_flush;
  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << "Before selection of contributing collinear 'dipoles'" << endline_flush;
  logger.newLine(LOG_DEBUG);

  selection_collinear();

  logger.newLine(LOG_DEBUG);
  logger << LOG_DEBUG << "After selection of contributing collinear 'dipoles'" << endline_flush;
  logger.newLine(LOG_DEBUG);
  
  logger << LOG_INFO << output_collinear() << endline_flush;
  logger << LOG_INFO << output_collinear_pdf() << endline_flush;
  logger << LOG_INFO << output_collinear_splitting() << endline_flush;
  logger << LOG_INFO << output_collinear_pa_name(temp_type_correction, pa_name) << endline_flush;
  logger.newLine(LOG_INFO);

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void observable_set::initialization_collinear_QEW(){
  Logger logger("observable_set::initialization_collinear_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  Kbar.resize(4, vector<double> (3, 0.));
  Kt.resize(4, vector<double> (3, 0.));
  P.resize(4, vector<double> (3, 0.));
  Kbar_plus.resize(4, vector<double> (3, 0.));
  Kt_plus.resize(4, vector<double> (3, 0.));
  P_plus.resize(4, vector<double> (3, 0.));
  intKbar_plus.resize(4, vector<double> (3, 0.));
  intKt_plus.resize(4, vector<double> (3, 0.));
  intP_plus.resize(4, vector<double> (3, 0.));
  Kbar_delta.resize(4, 0.);
  Kt_delta.resize(4, 0.);
  P_delta.resize(4, 0.);

  alpha_e_2pi = msi->alpha_e * inv2pi;

  gamma_i.resize((*CA_collinear).size());

  ln_papi.resize(3, vector<double> (csi->type_parton[0].size()));
  CA_value_ln_muF_papi.resize(CA_value_log_mu2_fact.size());
  value_dataP.resize(CA_value_log_mu2_fact.size());

  type.resize((*CA_collinear).size());
  no_emitter.resize((*CA_collinear).size());
  no_spectator.resize((*CA_collinear).size());
  collinear_singularity.resize((*CA_collinear).size());
  ppair.resize((*CA_collinear).size());

  // calculate all needed momentum-independent (splitting) functions
  if (CA_dipole_splitting[0][0] == 1 || CA_dipole_splitting[0][1] == 1 || CA_dipole_splitting[0][2] == 1){    // a -> a () splitting (0)
    // Was implemented only for massive bottom quark case before !!!
    Kbar_delta[0] = CS_QEW_Kbar_aa_delta;
    Kt_delta[0] = CS_QEW_Kt_aa_delta;
    P_delta[0] = CS_QEW_P_aa_delta;
  }

  if (CA_dipole_splitting[1][0] == 1 || CA_dipole_splitting[1][1] == 1 || CA_dipole_splitting[1][2] == 1){    // q -> q (a) splitting
    Kbar_delta[1] = CS_QEW_Kbar_qq_delta;
    Kt_delta[1] = CS_QEW_Kt_qq_delta;
    P_delta[1] = CS_QEW_P_qq_delta;
  }

  for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
    for (int j_c = 0; j_c < (*CA_collinear)[i_c].size(); j_c++){
      if ((*CA_collinear)[i_c][j_c].no_spectator() == 0){continue;}
      bool flag = false;
      for (int i_p = 0; i_p < pair.size(); i_p++){
	if ((*CA_collinear)[i_c][j_c].pair() == pair[i_p]){flag = true; break;}
      }
      if (!flag){pair.push_back((*CA_collinear)[i_c][j_c].pair());}
    }
  }

  for (int sd = 0; sd < CA_value_ln_muF_papi.size(); sd++){
    CA_value_ln_muF_papi[sd].resize(CA_value_log_mu2_fact[sd].size(), vector<vector<double> > (3, vector<double> (csi->type_parton[0].size())));
  }
  for (int sd = 0; sd < CA_value_ln_muF_papi.size(); sd++){
    value_dataP[sd].resize(CA_value_log_mu2_fact[sd].size(), vector<vector<vector<double> > > (3, vector<vector<double> > ((*CA_collinear).size())));
    for (int ss = 0; ss < CA_value_ln_muF_papi[sd].size(); ss++){
      for (int i_x = 0; i_x < 3; i_x++){
	for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
	  value_dataP[sd][ss][i_x][i_c].resize((*CA_collinear)[i_c].size(), 0.);
	}
      }
    }
  }

  for (int i_c = 0; i_c < gamma_i.size(); i_c++){ // i_c = -1 is not needed !!!
    gamma_i[i_c].resize((*CA_collinear)[i_c].size());
    for (int j_c = 0; j_c < (*CA_collinear)[i_c].size(); j_c++){
      // Refinement needed for hard on-shell photons etc. !!!
      if ((*CA_collinear)[i_c][j_c].type_parton()[(*CA_collinear)[i_c][j_c].no_spectator()] == 22){
	logger << LOG_WARN << "Does this ever happen ???" << endline_flush;
	gamma_i[i_c][j_c] = CS_QEW_gamma_a;
      }
      else {gamma_i[i_c][j_c] = CS_QEW_gamma_q;}
    }
  }

  for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
    type[i_c].resize((*CA_collinear)[i_c].size());
    no_emitter[i_c].resize((*CA_collinear)[i_c].size());
    no_spectator[i_c].resize((*CA_collinear)[i_c].size());
    collinear_singularity[i_c].resize((*CA_collinear)[i_c].size());
    ppair[i_c].resize((*CA_collinear)[i_c].size());
    for (int j_c = 0; j_c < (*CA_collinear)[i_c].size(); j_c++){
      type[i_c][j_c] = (*CA_collinear)[i_c][j_c].type();
      no_emitter[i_c][j_c] = (*CA_collinear)[i_c][j_c].no_emitter();
      no_spectator[i_c][j_c] = (*CA_collinear)[i_c][j_c].no_spectator();
      collinear_singularity[i_c][j_c] = (*CA_collinear)[i_c][j_c].in_collinear()[0];
      ppair[i_c][j_c] = (*CA_collinear)[i_c][j_c].pair();
    }
  }

  if (massive_QEW){
    P_reg.resize(4, vector<double> (3, 0.));
    
    gamma_ax.resize(3);
    for (int i_em = 1; i_em < 3; i_em++){
      if (csi->type_parton[0][i_em] == 22){gamma_ax[i_em] = CS_QEW_gamma_a;}
      else {gamma_ax[i_em] = CS_QEW_gamma_q;}
    }

    // Check meaning of n_max_spectator (looks a bit weird) !!!
    static int n_max_spectator = 0;
    for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
      if ((*CA_collinear)[i_c].size() > n_max_spectator){n_max_spectator = (*CA_collinear)[i_c].size();}
    }
    // Seems to be problematic if csi->type_parton[0].size() > n_max_spectator !!!
    if (n_max_spectator < csi->type_parton[0].size()){n_max_spectator = csi->type_parton[0].size();}
    // Probably this could be always used !!! ???

    Kit.resize(4, vector<vector<double> > (3, vector<double> (n_max_spectator, 0.)));
    intKit_plus.resize(4, vector<vector<double> > (3, vector<double> (n_max_spectator, 0.)));
    Kit_plus_x.resize(4, vector<vector<double> > (3, vector<double> (n_max_spectator, 0.)));
    Kit_plus_1.resize(4, vector<vector<double> > (3, vector<double> (n_max_spectator, 0.)));
    Kit_plus_outside_x.resize(4, vector<vector<double> > (3, vector<double> (n_max_spectator, 0.)));
    Kit_plus_outside_1.resize(4, vector<vector<double> > (3, vector<double> (n_max_spectator, 0.)));
    Kit_delta.resize(4, vector<vector<double> > (3, vector<double> (n_max_spectator, 0.)));
    m_Q.resize(csi->type_parton[0].size(), 0.);
    m2_Q.resize(csi->type_parton[0].size(), 0.);
    sall_ja.resize(3, vector<double> (csi->type_parton[0].size(), 0.));
    sall_ja_x.resize(3, vector<double> (csi->type_parton[0].size(), 0.));
    mu2_Q.resize(3, vector<double> (csi->type_parton[0].size(), 0.));
    mu2_Q_x.resize(3, vector<double> (csi->type_parton[0].size(), 0.));

    for (int i_p = 3; i_p < csi->type_parton[0].size(); i_p++){
      //      if (i_p < 3 && msi->M2_pdg[abs(csi->type_parton[0][i_p])] != 0.){logger << LOG_FATAL << "Incoming massive partons are not supported!" << endline_flush; int_end = 1; exit(1);}
      //      if (msi->M2_pdg[abs(csi->type_parton[0][i_p])] != 0.){
      if (csi->mass_parton[0][i_p] != 0.){
	m_Q[i_p] = csi->mass_parton[0][i_p];
	m2_Q[i_p] = csi->mass2_parton[0][i_p];
      }
    }

    for (int i_p = 0; i_p < pair.size(); i_p++){
      int pair_em = pair[i_p][0];
      int pair_sp = pair[i_p][1];
      // Shouldn't this situation throw an error message instead ???
      //      if (m2_Q[pair_em] > 0.){logger << LOG_FATAL << "Emitter may not be massive!" << endline_flush; exit(1);}
      if (m2_Q[pair_em] > 0.){return;}

      if (m2_Q[pair_sp] > 0.){
	for (int i_dt = 0; i_dt < 4; i_dt++){
	  Kit[i_dt][pair_em][pair_sp] = 0.;
	  Kit_plus_x[i_dt][pair_em][pair_sp] = 0.;
	  Kit_plus_1[i_dt][pair_em][pair_sp] = 0.;
	  intKit_plus[i_dt][pair_em][pair_sp] = 0.;
	  Kit_plus_outside_x[i_dt][pair_em][pair_sp] = 0.;
	  Kit_plus_outside_1[i_dt][pair_em][pair_sp] = 0.;
	  Kit_delta[i_dt][pair_em][pair_sp] = 0.;
	}
      }
    }
  }
 
  for (int i_p = 0; i_p < pair.size(); i_p++){logger << LOG_DEBUG << "pair[" << i_p << "] = " << "(" << pair[i_p][0] << ", " << pair[i_p][1] << ")" << endline_flush;}
    
  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}
  

void observable_set::calculate_collinear_QEW(){
  Logger logger("observable_set::calculate_collinear_QEW");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  static int initialization = 1;
  if (initialization == 1){
    initialization_collinear_QEW();
    
    initialization = 0;
    logger << LOG_DEBUG_VERBOSE << "initialization finished!" << endline_flush;
  }

  for (int i_em = 1; i_em < 3; i_em++){

    if (CA_dipole_splitting[0][i_em] == 1){      // a -> a () splitting
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[0][" << i_em << "] == 1   splitting kernels" << endline_flush;
      Kbar[0][i_em] = CS_QEW_Kbar_aa;  // 0
      Kt[0][i_em] = CS_QEW_Kt_aa;  // 0
      P[0][i_em] = CS_QEW_P_aa;  // 0
      if (massive_QEW){P_reg[0][i_em] = CS_QEW_P_aa_reg;}  // 0
      if (CA_dipole_splitting[0][0] == 1){
	Kbar_plus[0][i_em] = CS_QEW_Kbar_aa_plus;  // 0
	Kt_plus[0][i_em] = CS_QEW_Kt_aa_plus;  // 0
	P_plus[0][i_em] = CS_QEW_P_aa_plus;  // 0
	intKbar_plus[0][i_em] = CS_QEW_intKbar_aa_plus;  // 0
	intKt_plus[0][i_em] = CS_QEW_intKt_aa_plus;  // 0
	intP_plus[0][i_em] = CS_QEW_intP_aa_plus;  // 0
      }
      else {logger << LOG_FATAL << "May not happen !!! a -> a splitting without irregular terms !!!" << endline_flush; exit(1);}
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[0][" << i_em << "] == 1   splitting kernels done" << endline_flush;
    }
    else if (CA_dipole_splitting[1][i_em] == 1){      // q -> q (a) splitting
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[1][" << i_em << "] == 1   splitting kernels" << endline_flush;
      Kbar[1][i_em] = CS_QEW_Kbar_qq(psi->z_coll[i_em]);
      Kt[1][i_em] = CS_QEW_Kt_qq(psi->z_coll[i_em]);
      P[1][i_em] = CS_QEW_P_qq(psi->z_coll[i_em]);
      if (massive_QEW){P_reg[1][i_em] = CS_QEW_P_qq_reg(psi->z_coll[i_em]);}
      if (CA_dipole_splitting[1][0] == 1){
	Kbar_plus[1][i_em] = CS_QEW_Kbar_qq_plus(psi->z_coll[i_em]);
	Kt_plus[1][i_em] = CS_QEW_Kt_qq_plus(psi->z_coll[i_em]);
	P_plus[1][i_em] = CS_QEW_P_qq_plus(psi->z_coll[i_em]);
	intKbar_plus[1][i_em] = CS_QEW_intKbar_qq_plus(psi->x_pdf[i_em]);
	intKt_plus[1][i_em] = CS_QEW_intKt_qq_plus(psi->x_pdf[i_em]);
	intP_plus[1][i_em] = CS_QEW_intP_qq_plus(psi->x_pdf[i_em]);
      }
      else {logger << LOG_FATAL << "May not happen !!! q -> q splitting without irregular terms !!!" << endline_flush; exit(1);}
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[1][" << i_em << "] == 1   splitting kernels done" << endline_flush;
    }
    if (CA_dipole_splitting[2][i_em] == 1){      // q -> a (q) splitting ??? not checked yet !!!
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[2][" << i_em << "] == 1   splitting kernels" << endline_flush;
      Kbar[2][i_em] = CS_QEW_Kbar_qa(psi->z_coll[i_em]);
      Kt[2][i_em] = CS_QEW_Kt_qa(psi->z_coll[i_em]);
      P[2][i_em] = CS_QEW_P_qa(psi->z_coll[i_em]);
      if (massive_QEW){P_reg[2][i_em] = P[2][i_em];}
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[2][" << i_em << "] == 1   splitting kernels done" << endline_flush;
    }
    if (CA_dipole_splitting[3][i_em] == 1){      // a -> q (qx) splitting ??? not checked yet !!!
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[3][" << i_em << "] == 1   splitting kernels" << endline_flush;
      Kbar[3][i_em] = CS_QEW_Kbar_aq(psi->z_coll[i_em]);
      Kt[3][i_em] = CS_QEW_Kt_aq(psi->z_coll[i_em]);
      P[3][i_em] = CS_QEW_P_aq(psi->z_coll[i_em]);
      if (massive_QEW){P_reg[3][i_em] = P[3][i_em];}
      logger << LOG_DEBUG_VERBOSE << "CA_dipole_splitting[3][" << i_em << "] == 1   splitting kernels done" << endline_flush;
    }
  }


  // calculation of all Kbar, Kitalic, Ktilde, and P functions for KP terms:
  
  if (massive_QEW){calculate_collinear_QEW_CDST();}
  else {calculate_collinear_QEW_CS();}


  for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
    for (int i_x = 0; i_x < 3; i_x++){
      double temp_sumK = accumulate(data_K[i_x][i_c].begin(), data_K[i_x][i_c].end(), 0.);
      if (switch_TSV){
	for (int v_sf = 0; v_sf < max_dyn_fact + 1; v_sf++){
	  for (int v_xf = 0; v_xf < value_logscale2_fact_papi[v_sf].size(); v_xf++){
	    if (switch_KP == 0){
	      // K + P terms
	      value_ME2_KP[v_sf][v_xf][i_x][i_c] = alpha_e_2pi * (temp_sumK + accumulate(value_data_P[v_sf][v_xf][i_x][i_c].begin(), value_data_P[v_sf][v_xf][i_x][i_c].end(), 0.));
	      value_ME2term_fact[i_c][i_x][v_sf][v_xf] = alpha_e_2pi * (temp_sumK + accumulate(value_data_P[v_sf][v_xf][i_x][i_c].begin(), value_data_P[v_sf][v_xf][i_x][i_c].end(), 0.));
	    }
	    else if (switch_KP == 1){
	      // P terms
	      value_ME2_KP[v_sf][v_xf][i_x][i_c] = alpha_e_2pi * (accumulate(value_data_P[v_sf][v_xf][i_x][i_c].begin(), value_data_P[v_sf][v_xf][i_x][i_c].end(), 0.));
	      value_ME2term_fact[i_c][i_x][v_sf][v_xf] = alpha_e_2pi * (accumulate(value_data_P[v_sf][v_xf][i_x][i_c].begin(), value_data_P[v_sf][v_xf][i_x][i_c].end(), 0.));
	    }
	    else if (switch_KP == 2){
	      // K terms
	      value_ME2_KP[v_sf][v_xf][i_x][i_c] = alpha_e_2pi * (temp_sumK);
	      value_ME2term_fact[i_c][i_x][v_sf][v_xf] = alpha_e_2pi * (temp_sumK);
	    }
	  }
	}
      }

      for (int sd = 0; sd < CA_value_ln_muF_papi.size(); sd++){
	for (int ss = 0; ss < CA_value_ln_muF_papi[sd].size(); ss++){
	  if (switch_KP == 0){
	    // K + P term
	    CA_value_ME2_KP[sd][ss][i_x][i_c] = alpha_e_2pi * (temp_sumK + accumulate(value_dataP[sd][ss][i_x][i_c].begin(), value_dataP[sd][ss][i_x][i_c].end(), 0.));
	  }
	  else if (switch_KP == 1){
	    // P terms
	    CA_value_ME2_KP[sd][ss][i_x][i_c] = alpha_e_2pi * (accumulate(value_dataP[sd][ss][i_x][i_c].begin(), value_dataP[sd][ss][i_x][i_c].end(), 0.));
	  }
	  else if (switch_KP == 2){
	    // K terms
	    CA_value_ME2_KP[sd][ss][i_x][i_c] = alpha_e_2pi * (temp_sumK);
	  }
	}
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void observable_set::calculate_collinear_QEW_CS(){
  Logger logger("observable_set::calculate_collinear_QEW_CS");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  for (int i_p = 0; i_p < pair.size(); i_p++){
    ln_papi[pair[i_p][0]][pair[i_p][1]] = log(2 * esi->p_parton[0][pair[i_p][0]] * esi->p_parton[0][pair[i_p][1]]);
    if (switch_TSV){
      for (int v_sf = 0; v_sf < max_dyn_fact + 1; v_sf++){
	for (int v_xf = 0; v_xf < n_scale_dyn_fact[v_sf]; v_xf++){
	  value_logscale2_fact_papi[v_sf][v_xf][pair[i_p][0]][pair[i_p][1]] = value_central_logscale2_fact[v_sf] + value_relative_logscale2_fact[v_sf][v_xf] - ln_papi[pair[i_p][0]][pair[i_p][1]];
	}
      }
    }
    for (int sd = 0; sd < CA_value_ln_muF_papi.size(); sd++){
      for (int ss = 0; ss < CA_value_ln_muF_papi[sd].size(); ss++){
	CA_value_ln_muF_papi[sd][ss][pair[i_p][0]][pair[i_p][1]] = CA_value_log_mu2_fact[sd][ss] - ln_papi[pair[i_p][0]][pair[i_p][1]];
      }
    }
  }


  for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
    for (int j_c = 0; j_c < (*CA_collinear)[i_c].size(); j_c++){
      // K terms
      if (no_spectator[i_c][j_c] == 0){ // for (anti-)quarks, factor Q²_f included in the matrix element CA_ME2_cf[i_c][j_c] (which is Born * Q²_f) !!! ???
	if (switch_off_CS_ii || switch_off_CS_if){
	  data_K[0][i_c][j_c] = 0.;
	  data_K[1][i_c][j_c] = 0.;
	  data_K[2][i_c][j_c] = 0.;
	  double temp_charge2_emitter = (-1.); // -csi->charge2_parton[0][(*CA_collinear)[i_c][0].no_emitter()]
	  for (int k_c = 1; k_c < (*CA_collinear)[i_c].size(); k_c++){
	    if ((!switch_off_CS_ii && ((*CA_collinear)[i_c][k_c].no_emitter() < 3 && (*CA_collinear)[i_c][k_c].no_spectator() < 3)) ||
		(!switch_off_CS_if && ((*CA_collinear)[i_c][k_c].no_emitter() < 3 && (*CA_collinear)[i_c][k_c].no_spectator() > 2))){
	      data_K[0][i_c][j_c] += (Kbar[type[i_c][j_c]][no_emitter[i_c][j_c]] + Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][k_c] * temp_charge2_emitter;
	      data_K[1][i_c][j_c] += (-Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf[i_c][k_c] * temp_charge2_emitter;
	      data_K[2][i_c][j_c] += (Kbar_delta[type[i_c][j_c]] - intKbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][k_c] * temp_charge2_emitter;
	    }
	  }
	}
	else {
	  data_K[0][i_c][j_c] = (Kbar[type[i_c][j_c]][no_emitter[i_c][j_c]] + Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];
	  data_K[1][i_c][j_c] = (-Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf[i_c][j_c];
	  data_K[2][i_c][j_c] = (Kbar_delta[type[i_c][j_c]] - intKbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];
	}
      }
      else if (no_spectator[i_c][j_c] < 3){ // for (anti-)quarks, factor Q²_f always drops out due to T²_a->Q²_f in the denominators ???
	if (!switch_off_CS_ii){
	  data_K[0][i_c][j_c] = -(Kt[type[i_c][j_c]][no_emitter[i_c][j_c]] + Kt_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];
	  data_K[1][i_c][j_c] = -(-Kt_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf[i_c][j_c];
	  data_K[2][i_c][j_c] = -(Kt_delta[type[i_c][j_c]] - intKt_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];
	}
      }
      else if (no_spectator[i_c][j_c] > 2){ // for (anti-)quarks, factor Q²_f always drops out due to T²_a->Q²_f in the denominators ???
	// XXX // This contribution might have to vanish for an IS photon emitter and non-photon FS emitter since it is actually a remnant of a FS emitter contribution with IS spectator (set to 0 per convention used in sum kappa_ij,k = -1) !!!
	if (collinear_singularity[i_c][j_c] == 1){
	  // Using ..._fi charge factors:
	  data_K[0][i_c][j_c] = gamma_i[i_c][j_c] * (1. / (1. - psi->z_coll[no_emitter[i_c][j_c]])) * CA_ME2_cf_fi[i_c][j_c];
	  data_K[1][i_c][j_c] = -gamma_i[i_c][j_c] * (1. / (1. - psi->z_coll[no_emitter[i_c][j_c]])) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf_fi[i_c][j_c];
	  if (switch_off_intXplus){data_K[2][i_c][j_c] = gamma_i[i_c][j_c] * CA_ME2_cf_fi[i_c][j_c];}
	  else {data_K[2][i_c][j_c] = gamma_i[i_c][j_c] * (1. + log(1. - psi->x_pdf[no_emitter[i_c][j_c]])) * CA_ME2_cf_fi[i_c][j_c];}
	  // Charge factor Q²_i of final-state emitter was missing here !!! (also in Marek's paper ???)
	}
      }

      if (no_spectator[i_c][j_c] != 0){ // for (anti-)quarks, factor Q²_f always drops out due to T²_a->Q²_f in the denominators
	// P terms
	if ((!switch_off_CS_ii && ((*CA_collinear)[i_c][j_c].no_emitter() < 3 && (*CA_collinear)[i_c][j_c].no_spectator() < 3)) ||
	    (!switch_off_CS_if && ((*CA_collinear)[i_c][j_c].no_emitter() < 3 && (*CA_collinear)[i_c][j_c].no_spectator() > 2))){
	  if (switch_TSV){
	    for (int v_sf = 0; v_sf < value_logscale2_fact_papi.size(); v_sf++){
	      for (int v_xf = 0; v_xf < value_logscale2_fact_papi[v_sf].size(); v_xf++){
		value_data_P[v_sf][v_xf][0][i_c][j_c] = (P[type[i_c][j_c]][no_emitter[i_c][j_c]] + P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * value_logscale2_fact_papi[v_sf][v_xf][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
		value_data_P[v_sf][v_xf][1][i_c][j_c] = (-P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * value_logscale2_fact_papi[v_sf][v_xf][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
		value_data_P[v_sf][v_xf][2][i_c][j_c] = (P_delta[type[i_c][j_c]] - intP_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * value_logscale2_fact_papi[v_sf][v_xf][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	      }
	    }
	  }
	  for (int sd = 0; sd < CA_value_ln_muF_papi.size(); sd++){
	    for (int ss = 0; ss < CA_value_ln_muF_papi[sd].size(); ss++){
	      value_dataP[sd][ss][0][i_c][j_c] = (P[type[i_c][j_c]][no_emitter[i_c][j_c]] + P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_value_ln_muF_papi[sd][ss][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	      value_dataP[sd][ss][1][i_c][j_c] = (-P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_value_ln_muF_papi[sd][ss][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	      value_dataP[sd][ss][2][i_c][j_c] = (P_delta[type[i_c][j_c]] - intP_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_value_ln_muF_papi[sd][ss][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	    }
	  }
	}
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void observable_set::calculate_collinear_QEW_CDST(){
  static Logger logger("observable_set::calculate_collinear_QEW_CDST");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  for (int i_p = 0; i_p < pair.size(); i_p++){
    int pair_em = pair[i_p][0];
    int pair_sp = pair[i_p][1];
    // exception: 1 -- 2; however, in this case, for all relevant configurations the involved functions are symmetric!
    sall_ja[pair_em][pair_sp] = 2 * esi->p_parton[0][pair_em] * esi->p_parton[0][pair_sp];
    ln_papi[pair_em][pair_sp] = log(sall_ja[pair_em][pair_sp]);

    if (m2_Q[pair_sp] > 0.){
      // Only happens if no initial-initial dipole is discussed, i.e. pair_em and pair_sp really point at emitter and spectator, respectively.
      sall_ja_x[pair_em][pair_sp] = sall_ja[pair_em][pair_sp] / psi->z_coll[pair_em];
      mu2_Q[pair_em][pair_sp] = m2_Q[pair_sp] / sall_ja[pair_em][pair_sp];
      mu2_Q_x[pair_em][pair_sp] = m2_Q[pair_sp] / sall_ja_x[pair_em][pair_sp];

      // only for massive quarks as spectators
      for (int i_dt = 0; i_dt < 4; i_dt++){
	if      (i_dt == 0 && (CA_dipole_splitting[i_dt][1] == 1 || CA_dipole_splitting[i_dt][2] == 1)){
	  // a -> a (+a) splitting
	  // XXX // Check if this term should be here since there is no IS-photon emitter with a final-state spectator (always the other IS particle) ??? !!!
	  // XXX // It might be automatically switched off via dipole selection !!!
	  Kit[i_dt][pair_em][pair_sp] = - 2 * log(2. - psi->z_coll[pair_em]) / (1. - psi->z_coll[pair_em]); // from second term in (6.58) (-> K^qq_q) [included from (6.60)]
	  //	    + 2 * m2_Q[pair_sp] / (psi->z_coll[pair_em] * sall_ja_x[pair_em][pair_sp]) * log(m2_Q[pair_sp] / ((1. - psi->z_coll[pair_em]) * sall_ja_x[pair_em][pair_sp] + m2_Q[pair_sp])); // from (6.59) (-> K^qg_q) [included from (6.60)] // C_A -> 0
	  Kit_plus_x[i_dt][pair_em][pair_sp] =
	    + 2 * log(1. - psi->z_coll[pair_em]) / (1. - psi->z_coll[pair_em]) // from first term in (6.58)
	    + (1. - psi->z_coll[pair_em]) / (2 * pow(1. - psi->z_coll[pair_em] + mu2_Q_x[pair_em][pair_sp], 2)) // from first term in (5.58, J_gQ^a) included from (6.58)
	    - 2 / (1. - psi->z_coll[pair_em]) * (1. + log(1. - psi->z_coll[pair_em] + mu2_Q_x[pair_em][pair_sp])) // from second term in (5.58, J_gQ^a) included from (6.58)
	    ;
	  Kit_plus_1[i_dt][pair_em][pair_sp] =
	    + 2 * log(1. - psi->z_coll[pair_em]) / (1. - psi->z_coll[pair_em]) // from first term in (6.58)
	    + (1. - psi->z_coll[pair_em]) / (2 * pow(1. - psi->z_coll[pair_em] + mu2_Q[pair_em][pair_sp], 2)) // from first term in (5.58, J_gQ^a) included from (6.58)
	    - 2 / (1. - psi->z_coll[pair_em]) * (1. + log(1. - psi->z_coll[pair_em] + mu2_Q[pair_em][pair_sp])) // from second term in (5.58, J_gQ^a) included from (6.58)
	    ;
	  if (switch_off_intXplus){intKit_plus[i_dt][pair_em][pair_sp] = 0.;}
	  else {
	    intKit_plus[i_dt][pair_em][pair_sp] =
	      - pow(log(1. - psi->x_pdf[pair_em]), 2) // from first term in (6.58)
	      + .5 * (- mu2_Q[pair_em][pair_sp] / (1. - psi->x_pdf[pair_em] + mu2_Q[pair_em][pair_sp]) + mu2_Q[pair_em][pair_sp] / (1. + mu2_Q[pair_em][pair_sp]) - log((1. - psi->x_pdf[pair_em] + mu2_Q[pair_em][pair_sp]) / (1. + mu2_Q[pair_em][pair_sp]))) // from first term in (5.58) included from (6.58)
	      + 2 * (gsl_sf_dilog(-1. / mu2_Q[pair_em][pair_sp]) - gsl_sf_dilog(-(1. - psi->x_pdf[pair_em]) / mu2_Q[pair_em][pair_sp]) + log(1. - psi->x_pdf[pair_em]) * (1. + log(mu2_Q[pair_em][pair_sp]))) // from second term in (5.58) included from (6.58)
	      ;
	  }

	  // terms containing x-dependent pre-factor of (2/(1-psi->z_coll[pair_em]))_+
	  Kit_plus_outside_x[i_dt][pair_em][pair_sp] =
	    + log(((2. - psi->z_coll[pair_em]) * sall_ja_x[pair_em][pair_sp]) / ((2. - psi->z_coll[pair_em]) * sall_ja_x[pair_em][pair_sp] + m2_Q[pair_sp])) // from fourth term from (6.58)
	    + log(2. + mu2_Q_x[pair_em][pair_sp] - psi->z_coll[pair_em]) // from third term in (5.58) included from (6.58)
	    ;
	  Kit_plus_outside_1[i_dt][pair_em][pair_sp] =
	    + log(sall_ja[pair_em][pair_sp] / (sall_ja[pair_em][pair_sp] + m2_Q[pair_sp])) // from fourth term from (6.58)
	    + log(1. + mu2_Q[pair_em][pair_sp]) // from third term in (5.58) included from (6.58)
	    ;
	  // Kit_plus_outside_1[i_dt][pair_em][pair_sp]   seems to be identical to zero with the above definitions !!!
	  
	  Kit_delta[i_dt][pair_em][pair_sp] =
	    - CS_QEW_gamma_q // from fifth term from (6.58)
	    + mu2_Q[pair_em][pair_sp] * log(m2_Q[pair_sp] / (sall_ja[pair_em][pair_sp] + m2_Q[pair_sp])) // from sixth term from (6.58)
	    + .5 * m2_Q[pair_sp] / (sall_ja[pair_em][pair_sp] + m2_Q[pair_sp])// from seventh term from (6.58)
	    ;
	}

	if (i_dt == 1 && (CA_dipole_splitting[i_dt][1] == 1 || CA_dipole_splitting[i_dt][2] == 1)){
	  // q -> q (a) splitting
	  Kit[i_dt][pair_em][pair_sp] = -2 * log(2. - psi->z_coll[pair_em]) / (1. - psi->z_coll[pair_em]);
	  Kit_plus_x[i_dt][pair_em][pair_sp] = 0.
	    + 2 * log(1. - psi->z_coll[pair_em]) / (1. - psi->z_coll[pair_em]) // from first term in (6.58)
	    + (1. - psi->z_coll[pair_em]) / (2 * pow(1. - psi->z_coll[pair_em] + mu2_Q_x[pair_em][pair_sp], 2)) // from first term in (5.58, J_gQ^a) included from (6.58)
	    - 2 / (1. - psi->z_coll[pair_em]) * (1. + log(1. - psi->z_coll[pair_em] + mu2_Q_x[pair_em][pair_sp])) // from second term in (5.58, J_gQ^a) included from (6.58)
	    ;

	  Kit_plus_1[i_dt][pair_em][pair_sp] = 0.
	    + 2 * log(1. - psi->z_coll[pair_em]) / (1. - psi->z_coll[pair_em]) // from first term in (6.58)
	    + (1. - psi->z_coll[pair_em]) / (2 * pow(1. - psi->z_coll[pair_em] + mu2_Q[pair_em][pair_sp], 2)) // from first term in (5.58, J_gQ^a) included from (6.58)
	    - 2 / (1. - psi->z_coll[pair_em]) * (1. + log(1. - psi->z_coll[pair_em] + mu2_Q[pair_em][pair_sp])) // from second term in (5.58, J_gQ^a) included from (6.58)
	    ;

	  if (switch_off_intXplus){intKit_plus[i_dt][pair_em][pair_sp] = 0.;}
	  else {
	    intKit_plus[i_dt][pair_em][pair_sp] = 0.
	      - pow(log(1. - psi->x_pdf[pair_em]), 2) // from first term in (6.58)
	      + .5 * (- mu2_Q[pair_em][pair_sp] / (1. - psi->x_pdf[pair_em] + mu2_Q[pair_em][pair_sp]) + mu2_Q[pair_em][pair_sp] / (1. + mu2_Q[pair_em][pair_sp]) - log((1. - psi->x_pdf[pair_em] + mu2_Q[pair_em][pair_sp]) / (1. + mu2_Q[pair_em][pair_sp]))) // from first term in (5.58) included from (6.58)
	      + 2 * (gsl_sf_dilog(-1. / mu2_Q[pair_em][pair_sp]) - gsl_sf_dilog(-(1. - psi->x_pdf[pair_em]) / mu2_Q[pair_em][pair_sp]) + log(1. - psi->x_pdf[pair_em]) * (1. + log(mu2_Q[pair_em][pair_sp]))); // from second term in (5.58) included from (6.58)
	  }
	  
	  // terms containing x-dependent pre-factor of (2/(1-psi->z_coll[pair_em]))_+
	  Kit_plus_outside_x[i_dt][pair_em][pair_sp] =
	    + log(((2. - psi->z_coll[pair_em]) * sall_ja_x[pair_em][pair_sp]) / ((2. - psi->z_coll[pair_em]) * sall_ja_x[pair_em][pair_sp] + m2_Q[pair_sp])) // from fourth term from (6.58)
	    + log(2. + mu2_Q_x[pair_em][pair_sp] - psi->z_coll[pair_em]) // from third term in (5.58) included from (6.58)
	    ;
	  Kit_plus_outside_1[i_dt][pair_em][pair_sp] =
	    + log(sall_ja[pair_em][pair_sp] / (sall_ja[pair_em][pair_sp] + m2_Q[pair_sp])) // from fourth term from (6.58)
	    + log(1. + mu2_Q[pair_em][pair_sp]) // from third term in (5.58) included from (6.58)
	    ;
	  Kit_delta[i_dt][pair_em][pair_sp] =
	    - CS_QEW_gamma_q // from fifth term from (6.58)
	    + mu2_Q[pair_em][pair_sp] * log(m2_Q[pair_sp] / (sall_ja[pair_em][pair_sp] + m2_Q[pair_sp])) // from sixth term from (6.58)
	    + .5 * m2_Q[pair_sp] / (sall_ja[pair_em][pair_sp] + m2_Q[pair_sp]) // from seventh term from (6.58)
	    ;
	}
	else if (i_dt == 2 && (CA_dipole_splitting[i_dt][1] == 1 || CA_dipole_splitting[i_dt][2] == 1)){
	  // q -> a (q) splitting
	  Kit[i_dt][pair_em][pair_sp] =
	    2 * m2_Q[pair_sp] / (psi->z_coll[pair_em] * sall_ja_x[pair_em][pair_sp]) * log(m2_Q[pair_sp] / ((1. - psi->z_coll[pair_em]) * sall_ja_x[pair_em][pair_sp] + m2_Q[pair_sp])); // from (6.59) // C_F / C_A -> 1
	  //	    2 * 0.5 * m2_Q[pair_sp] / (psi->z_coll[pair_em] * sall_ja_x[pair_em][pair_sp]) * log(m2_Q[pair_sp] / ((1. - psi->z_coll[pair_em]) * sall_ja_x[pair_em][pair_sp] + m2_Q[pair_sp])); // from (6.59) // C_F / C_A -> 0.5 ???
	}
	else if (i_dt == 3 && (CA_dipole_splitting[i_dt][1] == 1 || CA_dipole_splitting[i_dt][2] == 1)){
	  // a -> q (qx) splitting
	  //	  Kit[i_dt][pair_em][pair_sp] = 0.;
	}
      }
    }

    if (switch_TSV){
      for (int v_sf = 0; v_sf < max_dyn_fact + 1; v_sf++){
	for (int v_xf = 0; v_xf < n_scale_dyn_fact[v_sf]; v_xf++){
	  value_logscale2_fact_papi[v_sf][v_xf][pair_em][pair_sp] = value_central_logscale2_fact[v_sf] + value_relative_logscale2_fact[v_sf][v_xf] - ln_papi[pair_em][pair_sp];
	}
      }
    }

    for (int sd = 0; sd < CA_value_ln_muF_papi.size(); sd++){
      for (int ss = 0; ss < CA_value_ln_muF_papi[sd].size(); ss++){
	CA_value_ln_muF_papi[sd][ss][pair_em][pair_sp] = CA_value_log_mu2_fact[sd][ss] - ln_papi[pair_em][pair_sp];
      }
    }
  }

  for (int i_c = 0; i_c < (*CA_collinear).size(); i_c++){
    for (int j_c = 0; j_c < (*CA_collinear)[i_c].size(); j_c++){
      // K terms
      if (no_spectator[i_c][j_c] == 0){
	if (switch_off_CS_ii || (switch_off_CS_if || switch_off_CS_if_massless || switch_off_CS_if_massive)){
	  data_K[0][i_c][j_c] = 0.;
	  data_K[1][i_c][j_c] = 0.;
	  data_K[2][i_c][j_c] = 0.;
	  double temp_charge2_emitter = (-1.);
	  for (int k_c = 1; k_c < (*CA_collinear)[i_c].size(); k_c++){
	    if ((!switch_off_CS_ii && ((*CA_collinear)[i_c][k_c].no_emitter() < 3 && (*CA_collinear)[i_c][k_c].no_spectator() < 3)) ||
		(!switch_off_CS_if && ((*CA_collinear)[i_c][k_c].no_emitter() < 3 && (*CA_collinear)[i_c][k_c].no_spectator() > 2) &&
		 ((!switch_off_CS_if_massless && !(*CA_collinear)[i_c][k_c].massive()) || (!switch_off_CS_if_massive && (*CA_collinear)[i_c][k_c].massive())))){
	      data_K[0][i_c][j_c] += (Kbar[type[i_c][j_c]][no_emitter[i_c][j_c]] + Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][k_c] * temp_charge2_emitter;
	      data_K[1][i_c][j_c] += (-Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf[i_c][k_c] * temp_charge2_emitter;
	      data_K[2][i_c][j_c] += (Kbar_delta[type[i_c][j_c]] - intKbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][k_c] * temp_charge2_emitter;
	    }
	  }
	}
	else {
	  data_K[0][i_c][j_c] = (Kbar[type[i_c][j_c]][no_emitter[i_c][j_c]] + Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];
	  data_K[1][i_c][j_c] = (-Kbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf[i_c][j_c];
	  data_K[2][i_c][j_c] = (Kbar_delta[type[i_c][j_c]] - intKbar_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];
	}
      }
      else if (no_spectator[i_c][j_c] < 3){
	if (!switch_off_CS_ii){
	  data_K[0][i_c][j_c] = -(Kt[type[i_c][j_c]][no_emitter[i_c][j_c]] + Kt_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];// reg
	  data_K[1][i_c][j_c] = -(-Kt_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf[i_c][j_c];// plus
	  data_K[2][i_c][j_c] = -(Kt_delta[type[i_c][j_c]] - intKt_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_ME2_cf[i_c][j_c];// delta
	}
      }
      else if (no_spectator[i_c][j_c] > 2){
	data_K[0][i_c][j_c] = 0.;
	data_K[1][i_c][j_c] = 0;
	data_K[2][i_c][j_c] = 0;
	if (m2_Q[no_spectator[i_c][j_c]] == 0. && !switch_off_CS_fi_massless){
	  if (!switch_off_CS_fi && ((!switch_off_CS_fi_massless && !(*CA_collinear)[i_c][j_c].massive()) || (!switch_off_CS_fi_massive && (*CA_collinear)[i_c][j_c].massive()))){
	    // remainder from final-initial dipoles -> Kit (use CA_ME2_cf_fi[i_c][j_c] instead of CA_ME2_cf[i_c][j_c] !)
	    if (collinear_singularity[i_c][j_c] == 1){
	      data_K[0][i_c][j_c] += gamma_i[i_c][j_c] * (1. / (1. - psi->z_coll[no_emitter[i_c][j_c]])) * CA_ME2_cf_fi[i_c][j_c];
	      data_K[1][i_c][j_c] += -gamma_i[i_c][j_c] * (1. / (1. - psi->z_coll[no_emitter[i_c][j_c]])) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf_fi[i_c][j_c];
   	      if (switch_off_intXplus){data_K[2][i_c][j_c] += gamma_i[i_c][j_c] * CA_ME2_cf_fi[i_c][j_c];}
	      else {data_K[2][i_c][j_c] += gamma_i[i_c][j_c] * (1. + log(1. - psi->x_pdf[no_emitter[i_c][j_c]])) * CA_ME2_cf_fi[i_c][j_c];}
	    }
	  }
	}
	if (m2_Q[no_spectator[i_c][j_c]] != 0. && !switch_off_CS_fi_massive){
	  if (!switch_off_CS_fi){
	    // remainder from final-initial dipoles -> Kit (use CA_ME2_cf_fi[i_c][j_c] instead of CA_ME2_cf[i_c][j_c] !)
	    data_K[0][i_c][j_c] += -(Kit[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]
				    + Kit_plus_x[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]
				    + 2. / (1. - psi->z_coll[no_emitter[i_c][j_c]]) * Kit_plus_outside_x[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]
				    ) * CA_ME2_cf_fi[i_c][j_c]; // reg
	    data_K[1][i_c][j_c] +=  -(- Kit_plus_1[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]
				     - 2. / (1. - psi->z_coll[no_emitter[i_c][j_c]]) * Kit_plus_outside_1[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]
				     ) * psi->z_coll[no_emitter[i_c][j_c]] * CA_ME2_cf_fi[i_c][j_c]; // plus
	    if (switch_off_intXplus){
	      data_K[2][i_c][j_c] += -(Kit_delta[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]) * CA_ME2_cf_fi[i_c][j_c]; // delta
	    }
	    else {
	      data_K[2][i_c][j_c] += -(Kit_delta[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]] // delta
				      - intKit_plus[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]
				      - 2 * log(1. - psi->x_pdf[no_emitter[i_c][j_c]]) * Kit_plus_outside_1[type[i_c][j_c]][no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]
				      ) * CA_ME2_cf_fi[i_c][j_c]; // delta
	    }
	  }
	}
	if (m2_Q[no_spectator[i_c][j_c]] != 0. && !switch_off_CS_if_massive){
	  if (!switch_off_CS_if){
	    // remainder from initial-final dipoles (only contributions from massive spectators)
	    // P -> P_reg (to be independent of convention; identical in the default convention)
	    data_K[0][i_c][j_c] += -P_reg[type[i_c][j_c]][no_emitter[i_c][j_c]] * log(((1. - psi->z_coll[no_emitter[i_c][j_c]]) * sall_ja_x[no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]) / ((1. - psi->z_coll[no_emitter[i_c][j_c]]) * sall_ja_x[no_emitter[i_c][j_c]][no_spectator[i_c][j_c]] + m2_Q[no_spectator[i_c][j_c]])) * CA_ME2_cf[i_c][j_c]; // from (6.55)
	    // no explizit plus term in (6.55)
	    if (collinear_singularity[i_c][j_c] == 1){
	      // contributes only to irregular splittings
	      data_K[2][i_c][j_c] += -gamma_ax[no_emitter[i_c][j_c]] * (log((sall_ja[no_emitter[i_c][j_c]][no_spectator[i_c][j_c]] - 2 * m_Q[no_spectator[i_c][j_c]] * sqrt(sall_ja[no_emitter[i_c][j_c]][no_spectator[i_c][j_c]] + m2_Q[no_spectator[i_c][j_c]]) + 2 * m2_Q[no_spectator[i_c][j_c]]) / sall_ja[no_emitter[i_c][j_c]][no_spectator[i_c][j_c]]) + 2 * m_Q[no_spectator[i_c][j_c]] / (sqrt(sall_ja[no_emitter[i_c][j_c]][no_spectator[i_c][j_c]] + m2_Q[no_spectator[i_c][j_c]]) + m_Q[no_spectator[i_c][j_c]])) * CA_ME2_cf[i_c][j_c];  // from (6.55)
	    }
	  }
	}
      }
      if (no_spectator[i_c][j_c] != 0){
	// P terms
	if ((!switch_off_CS_ii && ((*CA_collinear)[i_c][j_c].no_emitter() < 3 && (*CA_collinear)[i_c][j_c].no_spectator() < 3)) ||
	    (!switch_off_CS_if && ((*CA_collinear)[i_c][j_c].no_emitter() < 3 && (*CA_collinear)[i_c][j_c].no_spectator() > 2) &&
	     ((!switch_off_CS_if_massless && !(*CA_collinear)[i_c][j_c].massive()) || (!switch_off_CS_if_massive && (*CA_collinear)[i_c][j_c].massive())))){
	  if (switch_TSV){
	    for (int v_sf = 0; v_sf < value_logscale2_fact_papi.size(); v_sf++){
	      for (int v_xf = 0; v_xf < value_logscale2_fact_papi[v_sf].size(); v_xf++){
		value_data_P[v_sf][v_xf][0][i_c][j_c] = (P[type[i_c][j_c]][no_emitter[i_c][j_c]] + P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * value_logscale2_fact_papi[v_sf][v_xf][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
		value_data_P[v_sf][v_xf][1][i_c][j_c] = (-P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * value_logscale2_fact_papi[v_sf][v_xf][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
		value_data_P[v_sf][v_xf][2][i_c][j_c] = (P_delta[type[i_c][j_c]] - intP_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * value_logscale2_fact_papi[v_sf][v_xf][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	      }
	    }
	  }
	  for (int sd = 0; sd < CA_value_ln_muF_papi.size(); sd++){
	    for (int ss = 0; ss < CA_value_ln_muF_papi[sd].size(); ss++){
	      value_dataP[sd][ss][0][i_c][j_c] = (P[type[i_c][j_c]][no_emitter[i_c][j_c]] + P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_value_ln_muF_papi[sd][ss][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	      value_dataP[sd][ss][1][i_c][j_c] = (-P_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * psi->z_coll[no_emitter[i_c][j_c]] * CA_value_ln_muF_papi[sd][ss][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	      value_dataP[sd][ss][2][i_c][j_c] = (P_delta[type[i_c][j_c]] - intP_plus[type[i_c][j_c]][no_emitter[i_c][j_c]]) * CA_value_ln_muF_papi[sd][ss][ppair[i_c][j_c][0]][ppair[i_c][j_c][1]] * CA_ME2_cf[i_c][j_c];
	    }
	  }
	}
      }
    }
  }

  if (Log::getLogThreshold() <= LOG_DEBUG_POINT){output_calculate_collinear_QEW_CDST();}

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

