#include "header.hpp"

void observable_set::initialization_CS_splitting_coefficients_QEW(){
  static Logger logger("observable_set::calculate_splitting_coefficients");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  double sum_f_charge2 = 0.;
  for (int i_g = 1; i_g < 4; i_g++){
    // charged leptons
    if (msi->M_pdg[11 + (i_g - 1) * 2] == 0.){sum_f_charge2 += 1.;}
    // neutrinos
    if (msi->M_pdg[12 + (i_g - 1) * 2] == 0.){sum_f_charge2 += 0.;}
    // down-type quarks
    if (msi->M_pdg[1 + (i_g - 1) * 2] == 0.){sum_f_charge2 += N_c * (1. / 9.);}
    // up-type quarks
    if (msi->M_pdg[2 + (i_g - 1) * 2] == 0.){sum_f_charge2 += N_c * (4. / 9.);}
  }
  /*
  CS_gamma_a = -(2. / 3.) * sum_f_charge2;
  CS_K_a = -(10. / 9.) * sum_f_charge2;
  */
  // Identical content, just different names:
  CS_QEW_gamma_a = -(2. / 3.) * sum_f_charge2;
  CS_QEW_K_a = -(10. / 9.) * sum_f_charge2;

  CS_QEW_gamma_q = (3. / 2.); // C_F -> Q²_f
  CS_QEW_K_q = (7. / 2.) - pi2_6; // C_F -> Q²_f

  // Only K+P terms (CA):

  // P
  // Allow for different conventions in P_qq:
  if (switch_convention_P_qq){CS_QEW_P_qq_delta = (3. / 2.);}
  else {CS_QEW_P_qq_delta = 0.;}

  CS_QEW_P_aa_reg = 0.; // C_A -> 0
  CS_QEW_P_aa = 0.; // C_A -> 0
  CS_QEW_P_aa_plus = 0.; // C_A -> 0
  CS_QEW_intP_aa_plus = 0.; // C_A -> 0
  CS_QEW_P_aa_delta = (-2. / 3.) * sum_f_charge2; // C_A -> 0, T_R -> sum_f (N_c,f * Q²_f)

  // Kbar
  if (switch_convention_plus == 0){
    // original definition of +-distribution:
    CS_QEW_Kbar_qq_delta = -(5. - pi2); // C_F -> [Q²_q]
  }
  else if (switch_convention_plus == 1){
    // alternative definition of +-distribution:
    CS_QEW_Kbar_qq_delta = (-(5. - pi2) - pi2_3); // C_F -> [Q²_q]
  }
  else {exit(1);}
  //  else {logger << LOG_FATAL << "Undefined convention  switch_convention_plus = " << switch_convention_plus  << endline_flush; exit (1);}

  CS_QEW_Kbar_aa = 0.;
  CS_QEW_Kbar_aa_plus = 0.;
  CS_QEW_intKbar_aa_plus = 0.;
  CS_QEW_Kbar_aa_delta = (16. / 9.) * sum_f_charge2;

  // Kt
  CS_QEW_Kt_qq_delta = -pi2 / 3.; // C_F -> [Q²_q]

  CS_QEW_Kt_aa = 0.;
  CS_QEW_Kt_aa_plus = 0.;
  CS_QEW_intKt_aa_plus = 0.;
  CS_QEW_Kt_aa_delta = 0.;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

// P
// P_qa and P_aq have opposite N_c factors (compared to e.g. Marek's paper) !!!
double observable_set::CS_QEW_P_qa(double x){return (1. + pow(1. - x, 2)) / x;} // C_F -> [Q²_q]
double observable_set::CS_QEW_P_aq(double x){return N_c * (pow(x, 2) + pow(1. - x, 2));} // T_R -> N_c * [Q²_q]

double observable_set::CS_QEW_P_qq_reg(double x){return -(1. + x);} // C_F -> [Q²_q]
// Allow for different conventions via switch: affects P_qq (apart from P_qq_reg)
double observable_set::CS_QEW_P_qq(double x){
  if (switch_convention_P_qq){return -(1. + x);} // C_F -> [Q²_q]
  else {return 0.;}
}
double observable_set::CS_QEW_P_qq_plus(double x){
  if (switch_convention_P_qq){return 2. / (1. - x);} // C_F -> [Q²_q]
  else {return (1. + pow(x, 2)) / (1. - x);} // C_F -> [Q²_q]
}
double observable_set::CS_QEW_intP_qq_plus(double x){
  if (switch_off_intXplus){return 0.;}
  else {
    if (switch_convention_P_qq){return -2 * log(1. - x);} // C_F -> [Q²_q]
    else {return -.5 * pow(x, 2) - x - 2 * log(1. - x);} // C_F -> [Q²_q]
  }
}
//double observable_set::CS_QEW_P_qq_delta(double x){return CS_QEW_P_qq_delta;}

// Kbar
double observable_set::CS_QEW_Kbar_qa(double x){return CS_QEW_P_qa(x) * log((1. - x) / x) + x;} // C_F -> [Q²_q]
double observable_set::CS_QEW_Kbar_aq(double x){return CS_QEW_P_aq(x) * log((1. - x) / x) + 2 * N_c * x * (1. - x);} // T_R -> N_c * [Q²_q]

double observable_set::CS_QEW_Kbar_qq(double x){
  if (switch_convention_plus == 0){
    return (-(1. + x) * log((1. - x) / x) + (1. - x)); // C_F -> [Q²_q]
  }
  else if (switch_convention_plus == 1){
    return (-(1. + x) * log((1. - x) / x) + (1. - x) - 2 * log(x) / (1. - x)); // C_F -> [Q²_q]
  }
  else {exit(1);}
  //  else {logger << LOG_FATAL << "Undefined convention  switch_convention_plus = " << switch_convention_plus  << endline_flush;}
}
double observable_set::CS_QEW_Kbar_qq_plus(double x){
  if (switch_convention_plus == 0){
    return ((2 / (1. - x)) * log((1. - x) / x)); // C_F -> [Q²_q]
  }
  else if (switch_convention_plus == 1){
    return ((2. / (1. - x)) * log(1. - x)); // C_F -> [Q²_q]
  }
  else {exit(1);}
  //  else {logger << LOG_FATAL << "Undefined convention  switch_convention_plus = " << switch_convention_plus  << endline_flush;}
}
double observable_set::CS_QEW_intKbar_qq_plus(double x){
  if (switch_off_intXplus){return 0.;}
  else {
    if (switch_convention_plus == 0){
      return (-pow(log(1. - x), 2) - 2 * gsl_sf_dilog(1. - x) + pi2_3); // C_F -> [Q²_q]
    }
    else if (switch_convention_plus == 1){
      return (-pow(log(1. - x), 2)); // C_F -> [Q²_q]
    }
    else {exit(1);}
    //    else {logger << LOG_FATAL << "Undefined convention  switch_convention_plus = " << switch_convention_plus  << endline_flush;}
  }
}
//double observable_set::CS_QEW_Kbar_qq_delta(){return CS_QEW_Kbar_qq_delta;}

// original definition of +-distribution:
// alternative definition of +-distribution:
//!//double observable_set::CS_QEW_Kbar_qq_plus(double x){return ((2. / (1. - x)) * log(1. - x));} // C_F -> [Q²_q]
//!//double observable_set::CS_QEW_Kbar_qq(double x){return (-(1. + x) * log((1. - x) / x) + (1. - x) - 2 * log(x) / (1. - x));} // C_F -> [Q²_q]
//double observable_set::CS_QEW_Kbar_qq_delta(){return CS_QEW_Kbar_qq_delta;}
//!///////  double observable_set::CS_QEW_intKbar_qq_plus(double x){return (-pow(log(1. - x), 2));} // C_F -> [Q²_q]
//!//double observable_set::CS_QEW_intKbar_qq_plus(double x){return 0.;}


// Kt
double observable_set::CS_QEW_Kt_qa(double x){return CS_QEW_P_qa(x) * log(1. - x);} // C_F -> [Q²_f]
double observable_set::CS_QEW_Kt_aq(double x){return CS_QEW_P_aq(x) * log(1. - x);} // T_R -> N_c * [Q²_f]

double observable_set::CS_QEW_Kt_qq(double x){return CS_QEW_P_qq_reg(x) * log(1. - x);} // C_F -> [Q²_q]
double observable_set::CS_QEW_Kt_qq_plus(double x){return (2 / (1. - x)) * log(1. - x);} // C_F -> [Q²_q]
double observable_set::CS_QEW_intKt_qq_plus(double x){
  if (switch_off_intXplus){return 0.;}
  else {
    return -pow(log(1. - x), 2); // C_F -> [Q²_q]
  }
}
//double observable_set::CS_QEW_Kt_qq_delta(){return CS_QEW_Kt_qq_delta;}


