#include "header.hpp"

xdistribution::xdistribution(){
  static Logger logger("xdistribution");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  xdistribution_name = "";
  xdistribution_type = "";
  all_particle.push_back(vector<int> (0));
  all_particle_group.push_back(vector<int> (0));
  required_subparticle.push_back(vector<int> (0));
  required_particle.push_back(0);
  n_bins = 0;
  start = 0.;
  end = 0.;
  step = 0.;
  string edges = "";
  symm = 0;

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

xdistribution::xdistribution(string _xdistribution_name, string _xdistribution_type, vector<vector<int> > _all_particle, vector<vector<int> > _all_particle_group, vector<vector<int> > _required_subparticle, double _start, double _end, int _n_bins, double _step, string _edges, string _type_binning, observable_set *_osi){
  static Logger logger("xdistribution::xdistribution");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  xdistribution_name = _xdistribution_name;
  xdistribution_type = _xdistribution_type;
  osi = _osi;
  all_particle = _all_particle;
  all_particle_group = _all_particle_group;
  required_subparticle = _required_subparticle;

  initialization_particle();
  initialization_type_cumulative();

  reconstructedParticles.resize(particles.size());
  
  logger << LOG_DEBUG << "xdistribution_name = " << _xdistribution_name << endline_flush;
  logger << LOG_DEBUG << "required_subparticle.size() = " << required_subparticle.size() << endline_flush;
  logger << LOG_DEBUG << "required_particle.size() = " << required_particle.size() << endline_flush;
  required_particle.resize(required_subparticle.size());
  for (int i_p = 0; i_p < required_particle.size(); i_p++){
    int flag = 0;
    logger << LOG_DEBUG << "required_subparticle[" << i_p << "].size() = " << required_subparticle[i_p].size() << endline_flush;
    for (int i_s = 0; i_s < required_subparticle[i_p].size(); i_s++){
      if (required_subparticle[i_p][i_s] >= 0){flag = GSL_MIN(1, required_subparticle[i_p][i_s]); break;}
    }
    if (flag > 0){required_particle[i_p] = flag;}
  logger << LOG_DEBUG << "required_particle[" << i_p << "] = " << required_particle[i_p] << endline_flush;
  }
  logger << LOG_DEBUG << "done!" << endline_flush;
  
  initialization_range(_type_binning, _n_bins, _start, _end, _step, _edges);
  initialization_symmetry();
  
  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


void xdistribution::determineBin() {
  static Logger logger("xdistribution::determineBin");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  observable.resize(osi->n_ps);
  observable_max.resize(osi->n_ps);
  bin.resize(osi->n_ps);
  bin_max.resize(osi->n_ps);

  for (int i_a = 0; i_a < osi->n_ps; i_a++) {
    if (osi->esi->cut_ps[i_a] == -1) {
      // special cases should only be needed for backward compatibility; can be removed once old implementation has been removed
      if (xdistribution_type!="XS" && xdistribution_type!="multiplicity") {
//         bin[i_a]=0;
        continue;
      }
    }

    // make sure that all required particles exist, otherwise don't bin
    if (!checkRequirements(i_a)) {
      bin[i_a] = -1;
      bin_max[i_a] = -1;

      if (xdistribution_type == "pTveto" || xdistribution_type == "pTvetoexcl") {
        // nothing needs to be vetoed!
        bin[i_a]=0;
        bin_max[i_a] = n_bins;
      }

      continue;
    }

    fillMomenta(i_a);
    int save_RA_x_a = osi->RA_x_a;
    osi->RA_x_a = i_a;
    computeObservable(observable[i_a], observable_max[i_a]);
    osi->RA_x_a = save_RA_x_a;

    // why double ???
    double _bin = -1;
    double _bin_max = -1;


    if (typeCumulative == CUMULATIVE_NONE){
      _bin = performBinning(observable[i_a]);
      if (_bin < 0){_bin = -1;}
      else if (_bin >= n_bins){_bin = -1;}
    }
    else if (typeCumulative == CUMULATIVE_LOWER){
      _bin = 1 + performBinning(observable[i_a]);
      if (_bin >= n_bins){_bin = -1;}
      if (_bin >= 0){_bin_max = n_bins;}
      else {_bin = -1; _bin_max = -1;}
    }
    else if (typeCumulative == CUMULATIVE_UPPER) {
      _bin_max = 1+performBinning(observable_max[i_a]);
      _bin_max = min(_bin_max,double(n_bins));
      if (_bin_max >= 0) {
        _bin = 0;
      } else {
        _bin = -1;
        _bin_max = -1;
      }
    } else if (typeCumulative == CUMULATIVE_BOTH) {
      _bin = 1+performBinning(observable[i_a]);
      _bin_max = 1+performBinning(observable_max[i_a]);
      if (_bin >= n_bins) {
        _bin = -1;
      }
      if (_bin >= 0 && _bin_max >= 0) {
        _bin_max = min(_bin_max,double(n_bins));
      } else {
        _bin = -1;
        _bin_max = -1;
      }
    }

    bin[i_a] = int(_bin);
    bin_max[i_a] = int(_bin_max);

    if (bin[i_a]<-1) {
      cout << bin[i_a] << ", " << _bin << endl;
    }

    //logger << LOG_DEBUG_POINT << xdistribution_name << ", " << i_a << ": " << bin[i_a] << endline_flush;
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}


bool xdistribution::checkRequirements(int i_a){
  static Logger logger("xdistribution::checkRequirements");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  // For some special cases, this check should always return true:
  if (xdistribution_type == "multiplicity"){logger << LOG_DEBUG_VERBOSE << "xdistribution_type = " << xdistribution_type << " : return true" << endline_flush; return true;}

  bool checkRequirements_passed = true;
  for (int i_g = 0; i_g < particles.size(); i_g++){
    for (int i_p = 0; i_p < particles[i_g].size(); i_p++){
      if (particles[i_g][i_p].required_definition){if (particles[i_g][i_p].number > osi->esi->n_object[particles[i_g][i_p].type][i_a]){checkRequirements_passed = false; break;}}
      if (particles[i_g][i_p].required_exists){if (particles[i_g][i_p].number > osi->esi->particle_event[particles[i_g][i_p].type][i_a].size()){checkRequirements_passed = false; break;}}
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished - return checkRequirements_passed = " << checkRequirements_passed << endline_flush;
  return checkRequirements_passed;
}

/*
bool xdistribution::checkRequirements(int i_a) {
  static Logger logger("xdistribution::checkRequirements");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  // special cases
  if (xdistribution_type == "multiplicity")
    return true;
  
  for (int group=0; group<particles.size(); group++) {
    for (int particle=0; particle<particles[group].size(); particle++) {
      if (particles[group][particle].required_definition) {
        if (particles[group][particle].number > osi->esi->n_object[particles[group][particle].type][i_a]) {
          return false;
        }
      } else if (particles[group][particle].required_exists) {
        if (particles[group][particle].number > osi->esi->particle_event[particles[group][particle].type][i_a].size()) {
          return false;
        }
      }
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished - return " << true << endline_flush;
  return true;
}
*/


void xdistribution::fillMomenta(int i_a) {
  static Logger logger("xdistribution::fillMomenta");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  for (int group=0; group<particles.size(); group++) {
    reconstructedParticles[group] = nullvector;
    for (int particle=0; particle<particles[group].size(); particle++) {
      if (particles[group][particle].number <= osi->esi->particle_event[particles[group][particle].type][i_a].size() && particles[group][particle].number>=1) {
        particles[group][particle].momentum = osi->esi->particle_event[particles[group][particle].type][i_a][particles[group][particle].number-1].momentum;
        particles[group][particle].is_present = true;
      } else {
        particles[group][particle].momentum = fourvector(0.0,0.0,0.0,0.0);
        particles[group][particle].is_present = false;
      }
      particles[group][particle].multiplicity = osi->esi->n_object[particles[group][particle].type][i_a];

      reconstructedParticles[group] = reconstructedParticles[group] + particles[group][particle].momentum;
    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}

double xdistribution::performBinning(double value) {
  static Logger logger("xdistribution::performBinning");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  double bin = -1;

  if (type_binning == "linear"){
    bin = (value - start) / step;
  }
  else if (type_binning == "logarithmic"){
    if (value <= 0.){return -1;}
    else {bin = (log10(value) - start) / step;}
  }
  else if (type_binning == "irregular"){
    if (value < bin_edge[0] || value > bin_edge[n_bins]){
      return -1;
    }
    else {
      for (int i_b = 1; i_b < n_bins + 1; i_b++){
	if (value < bin_edge[i_b]){
	  bin = i_b - 1;
	  break;
	}
      }
    }
  }
  else {
    assert(false);
    return -1;
  }

  if (munich_isinf(bin)) {
    logger << LOG_DEBUG_POINT << xdistribution_name << ": bin is inf, bin=" << bin << ", value=" << value << endline_flush;
    bin = -1;
  }
  if (munich_isnan(bin)) {
    logger << LOG_DEBUG_POINT << xdistribution_name << ": bin is inf/nan, bin=" << bin << ", value=" << value << endline_flush;
    //    logger << LOG_WARN << xdistribution_name << ": bin is inf/nan, bin=" << bin << ", value=" << value << endline_flush;
    bin = -1;
  }

  /*
  // check for absurdly large bin values
  if (xdistribution_name == "absetamaxvetopTlep20_lep"){
    cout << xdistribution_name << "   bin = " << bin << endl;
    cout << xdistribution_name << "   (double)std::numeric_limits<int>::max() = " << (double)std::numeric_limits<int>::max() << endl;
    cout << xdistribution_name << "   (double)std::numeric_limits<int>::min() = " << (double)std::numeric_limits<int>::min() << endl;
    //    cout << xdistribution_name << "   bin = " << bin << endl;
  }
  if (bin > (double)std::numeric_limits<int>::max() || bin < (double)std::numeric_limits<int>::min()) {
    //    logger << LOG_DEBUG_VERBOSE << xdistribution_name << ": bin too large, bin=" << bin << ", value=" << value << endline_flush;
    bin = -1;
  }
  */

  logger << LOG_DEBUG_VERBOSE << "finished - " << xdistribution_name << " - bin = " << bin << endline_flush;
  return bin;
}


ostream & operator << (ostream &s, const xdistribution & sp){
  s << setw(25) << sp.xdistribution_name << "   ";
  s << "type: " << setw(15) << sp.xdistribution_type << "   ";
  s << "interval: " << setw(5) << sp.start << " - " << setw(4) << sp.step << "(" << setw(3) << sp.n_bins << ") - " << setw(5) << sp.end << "   ";
  s << "particles: ";
  for (int i_p = 0; i_p < (sp.all_particle).size(); i_p++){
    s << "[";
    for (int i_s = 0; i_s < (sp.all_particle[i_p]).size(); i_s++){
      s << sp.all_particle[i_p][i_s] << "(" << sp.all_particle_group[i_p][i_s] << ", ";
      if (sp.required_subparticle[i_p][i_s] == -1){s << "-";}
      else if (sp.required_subparticle[i_p][i_s] == 0){s << "0";}
      else if (sp.required_subparticle[i_p][i_s] == 1){s << "+";}
      s << ")";
      if (i_s < sp.all_particle[i_p].size() - 1){s << ", ";}
    }
    s << ", ";
    if (sp.required_particle[i_p] == 0){s << "-";}
    //    if (sp.required_particle[i_p] == -1){s << "-";}
    //    else if (sp.required_particle[i_p] == 0){s << "0";}
    else if (sp.required_particle[i_p] == 1){s << "+";}
    s << "]";

    if (i_p < sp.all_particle.size() - 1){s << " - ";}
  }
  s << "   ";
  return s;
}


void xdistribution::initialization_distribution_bin(){
  static Logger logger("xdistribution::initialization_distribution_bin");
  logger << LOG_DEBUG_VERBOSE << "called" << endline_flush;

  distribution_no_bin_phasespace.clear();
  distribution_no_bin_no_qTcut_phasespace.clear();

  if (osi->n_ps == 1){
    logger << LOG_DEBUG_VERBOSE << xdistribution_name << "   CASE   osi->n_ps == 1" << endline_flush;

    if (typeCumulative != CUMULATIVE_NONE){
      logger << LOG_DEBUG_VERBOSE << xdistribution_name << "   CASE   osi->n_ps == 1 && typeCumulative != CUMULATIVE_NONE" << endline_flush;
      distribution_no_bin = bin; // could still be -1
      distribution_no_bin_max = bin_max; // could still be -1
      if (bin[0] != -1 && bin_max[0] != -1){
	distribution_no_bin_phasespace = vector<vector<int> > (1, vector<int> (1, 0));
	///	distribution_no_bin_phasespace.resize(1, vector<int> (1, 0)); // -> (1, 0) only 1 phasespace, namely 0; resize(1, ...) -> only 1 bin
	/////	    distribution_no_qTcut_phasespace.resize(1, vector<int> (1, 0)); // -> (1, 0) only 1 phasespace value, namely 0; resize(1, ...) -> only 1 qTcut value

	distribution_no_bin_no_qTcut_phasespace = vector<vector<vector<int> > > (1, vector<vector<int> > (1, vector<int> (1, 0)));
	///	distribution_no_bin_no_qTcut_phasespace.resize(1, vector<vector<int> > (1, vector<int> (1, 0)));
      }
    }

    else if (typeCumulative == CUMULATIVE_NONE){
      logger << LOG_DEBUG_VERBOSE << xdistribution_name << "   CASE   osi->n_ps == 1 && typeCumulative == CUMULATIVE_NONE" << endline_flush;

      if (bin[0] != -1){
	distribution_no_bin = bin; // could still be -1
	// why this -> ???    distribution_no_bin.resize(1, bin[0]);
	// not needed:   distribution_no_bin_max = bin_max; // could still be -1
	distribution_no_bin_phasespace = vector<vector<int> > (1, vector<int> (1, 0));
	///	distribution_no_bin_phasespace.resize(1, vector<int> (1, 0)); // -> (1, 0) only 1 phasespace, namely 0; resize(1, ...) -> only 1 bin
	distribution_no_bin_no_qTcut_phasespace = vector<vector<vector<int> > > (1, vector<vector<int> > (1, vector<int> (1, 0)));
	///	distribution_no_bin_no_qTcut_phasespace.resize(1, vector<vector<int> > (1, vector<int> (1, 0)));
      }
    }

    else {
      logger << LOG_FATAL << "Wrong  typeCumulative  set!" << endline_flush; exit(1);
    }
  }

  else if (osi->n_ps > 1){
    logger << LOG_DEBUG_VERBOSE << xdistribution_name << "   CASE   osi->n_ps > 1" << endline_flush;

    if (typeCumulative != CUMULATIVE_NONE){
      logger << LOG_DEBUG_VERBOSE << xdistribution_name << "   CASE   osi->n_ps > 1 && typeCumulative != CUMULATIVE_NONE" << endline_flush;

      // starting point: distribution_no_bin   contains a vector<int> (osi->n_ps) with the bin values for each phase-space as entries
      distribution_no_bin = bin;
      distribution_no_bin_max = bin_max;

      sort(distribution_no_bin.begin(), distribution_no_bin.end());
      for (int i_b = osi->n_ps - 1; i_b > 0; i_b--){
	if (distribution_no_bin[i_b] == distribution_no_bin[i_b - 1]){distribution_no_bin.erase(distribution_no_bin.begin() + i_b);}
      }
      if (distribution_no_bin[0] == -1){distribution_no_bin.erase(distribution_no_bin.begin());}

      sort(distribution_no_bin_max.begin(), distribution_no_bin_max.end());
      for (int i_b = osi->n_ps - 1; i_b > 0; i_b--){
	if (distribution_no_bin_max[i_b] == distribution_no_bin_max[i_b - 1]){distribution_no_bin_max.erase(distribution_no_bin_max.begin() + i_b);}
      }
      if (distribution_no_bin_max[0] == -1){distribution_no_bin_max.erase(distribution_no_bin_max.begin());}

      distribution_no_bin_all = distribution_no_bin;
      for (int i_b = 0; i_b < distribution_no_bin_max.size(); i_b++){distribution_no_bin_all.push_back(distribution_no_bin_max[i_b]);}
      sort(distribution_no_bin_all.begin(), distribution_no_bin_all.end());
      for (int i_b = distribution_no_bin_all.size() - 1; i_b > 0; i_b--){
	if (distribution_no_bin_all[i_b] == distribution_no_bin_all[i_b - 1]){distribution_no_bin_all.erase(distribution_no_bin_all.begin() + i_b);}
      }

      stringstream out_dnb;
      for (int i_b = 0; i_b < distribution_no_bin.size(); i_b++){out_dnb << distribution_no_bin[i_b] << "   ";}
      logger << LOG_DEBUG << "out_dnb [" << xdistribution_name << "] = " << out_dnb.str() << endline_flush;
      stringstream out_dnbx;
      for (int i_b = 0; i_b < distribution_no_bin_max.size(); i_b++){out_dnbx << distribution_no_bin_max[i_b] << "   ";}
      logger << LOG_DEBUG << "out_dnbx[" << xdistribution_name << "] = " << out_dnbx.str() << endline_flush;
      stringstream out_dnba;
      for (int i_b = 0; i_b < distribution_no_bin_all.size(); i_b++){out_dnba << distribution_no_bin_all[i_b] << "   ";}
      logger << LOG_DEBUG << "out_dnba[" << xdistribution_name << "] = " << out_dnba.str() << endline_flush;

      distribution_no_bin_phasespace.resize(distribution_no_bin_all.size());
      for (int i_a = 0; i_a < osi->n_ps; i_a++){
	for (int i_b = 0; i_b < distribution_no_bin_all.size(); i_b++){
	  // needs modification for multi-differential distributions:
	  if (distribution_no_bin_all[i_b] >= bin[i_a] && distribution_no_bin_all[i_b] < bin_max[i_a]){distribution_no_bin_phasespace[i_b].push_back(i_a);}
	  if (distribution_no_bin_all[i_b] == bin_max[i_a]){break;}
	}
      }

      for (int x_b = 0; x_b < distribution_no_bin.size(); x_b++){
	stringstream out_dnbp;
	for (int x_a = 0; x_a < distribution_no_bin_phasespace[x_b].size(); x_a++){
	  out_dnbp << distribution_no_bin_phasespace[x_b][x_a] << " ";
	}
	logger << LOG_DEBUG << "out_dnb_phasespace [" << xdistribution_name << "][" << x_b << "] @ " << setw(3) << distribution_no_bin_all[x_b] << " = " << out_dnbp.str() << endline_flush;
      }

      if (distribution_no_bin_all.size() != 0){
	distribution_no_bin_no_qTcut_phasespace.resize(distribution_no_bin_all.size() - 1, vector<vector<int> > (osi->distribution_no_qTcut.size()));
	for (int i_b = 0; i_b < distribution_no_bin_all.size() - 1; i_b++){ // -1, because last entry contains no dipoles numbers any more
	  for (int i_q = 0; i_q < osi->distribution_no_qTcut.size(); i_q++){
	    for (int i_ba = 0; i_ba < distribution_no_bin_phasespace[i_b].size(); i_ba++){
	      for (int i_qa = 0; i_qa < osi->distribution_no_qTcut_phasespace[i_q].size(); i_qa++){
		if (distribution_no_bin_phasespace[i_b][i_ba] == osi->distribution_no_qTcut_phasespace[i_q][i_qa]){
		  distribution_no_bin_no_qTcut_phasespace[i_b][i_q].push_back(distribution_no_bin_phasespace[i_b][i_ba]);
		}
	      }
	    }
	  }
	}
      }
    }

    else if (typeCumulative == CUMULATIVE_NONE){
      logger << LOG_DEBUG_VERBOSE << xdistribution_name << "   CASE   osi->n_ps > 1 && typeCumulative == CUMULATIVE_NONE" << endline_flush;

      distribution_no_bin = bin;
      sort(distribution_no_bin.begin(), distribution_no_bin.end());

      // check why n_ps - 1 !!!
      for (int i_b = osi->n_ps - 1; i_b > 0; i_b--){
	if (distribution_no_bin[i_b] == distribution_no_bin[i_b - 1]){distribution_no_bin.erase(distribution_no_bin.begin() + i_b);}
      }
      if (distribution_no_bin[0] == -1){distribution_no_bin.erase(distribution_no_bin.begin());}

      distribution_no_bin_phasespace.resize(distribution_no_bin.size());
      for (int i_a = 0; i_a < osi->n_ps; i_a++){
	for (int i_b = 0; i_b < distribution_no_bin.size(); i_b++){
	  if (distribution_no_bin[i_b] == bin[i_a]){distribution_no_bin_phasespace[i_b].push_back(i_a); break;}
	}
      }

      distribution_no_bin_no_qTcut_phasespace.resize(distribution_no_bin.size(), vector<vector<int> > (osi->distribution_no_qTcut.size()));
      for (int i_b = 0; i_b < distribution_no_bin.size(); i_b++){
	for (int i_q = 0; i_q < osi->distribution_no_qTcut.size(); i_q++){
	  for (int i_ba = 0; i_ba < distribution_no_bin_phasespace[i_b].size(); i_ba++){
	    for (int i_qa = 0; i_qa < osi->distribution_no_qTcut_phasespace[i_q].size(); i_qa++){
	      if (distribution_no_bin_phasespace[i_b][i_ba] == osi->distribution_no_qTcut_phasespace[i_q][i_qa]){
		distribution_no_bin_no_qTcut_phasespace[i_b][i_q].push_back(distribution_no_bin_phasespace[i_b][i_ba]);
	      }
	    }
	  }
	}
      }

    }
  }

  logger << LOG_DEBUG_VERBOSE << "finished" << endline_flush;
}
